# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Interceptor that adds headers to outgoing requests."""

import collections

import grpc
import immudb.generic_client_interceptor as generic_client_interceptor
import immudb.grpc.schema_pb2_grpc as schema_pb2_grpc


class _ClientCallDetails(
        collections.namedtuple(
            '_ClientCallDetails',
            ('method', 'timeout', 'metadata', 'credentials')),
        grpc.ClientCallDetails):
    pass


def timeout_adder_interceptor(stubTimeout=None):

    def intercept_call(client_call_details, request_iterator, request_streaming,
                       response_streaming):
        timeoutToSet = client_call_details.timeout
        if(timeoutToSet == None):
            timeoutToSet = stubTimeout
        client_call_details = _ClientCallDetails(
            client_call_details.method, timeoutToSet, client_call_details.metadata,
            client_call_details.credentials)
        return client_call_details, request_iterator, None

    return generic_client_interceptor.create(intercept_call)


def header_adder_interceptor(header, value):

    def intercept_call(client_call_details, request_iterator, request_streaming,
                       response_streaming):
        metadata = []
        if client_call_details.metadata is not None:
            metadata = list(client_call_details.metadata)
        metadata.append((
            header,
            value,
        ))
        client_call_details = _ClientCallDetails(
            client_call_details.method, client_call_details.timeout, metadata,
            client_call_details.credentials)
        return client_call_details, request_iterator, None

    return generic_client_interceptor.create(intercept_call)


def get_intercepted_stub(channel, headers):
    try:
        intercepted = grpc.intercept_channel(channel, *headers)
        newStub = schema_pb2_grpc.ImmuServiceStub(intercepted)
        return intercepted, newStub
    except ValueError as e:
        raise Exception(
            "Attempted to login on termninated client, channel has been shutdown") from e
