# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: schema.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='schema.proto',
  package='immudb.schema',
  syntax='proto3',
  serialized_options=_b('Z+github.com/codenotary/immudb/pkg/api/schema'),
  serialized_pb=_b('\n\x0cschema.proto\x12\rimmudb.schema\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1cgoogle/protobuf/struct.proto\"\x12\n\x03Key\x12\x0b\n\x03key\x18\x01 \x01(\x0c\"2\n\nPermission\x12\x10\n\x08\x64\x61tabase\x18\x01 \x01(\t\x12\x12\n\npermission\x18\x02 \x01(\r\"z\n\x04User\x12\x0c\n\x04user\x18\x01 \x01(\x0c\x12.\n\x0bpermissions\x18\x03 \x03(\x0b\x32\x19.immudb.schema.Permission\x12\x11\n\tcreatedby\x18\x04 \x01(\t\x12\x11\n\tcreatedat\x18\x05 \x01(\t\x12\x0e\n\x06\x61\x63tive\x18\x06 \x01(\x08\".\n\x08UserList\x12\"\n\x05users\x18\x01 \x03(\x0b\x32\x13.immudb.schema.User\"Y\n\x11\x43reateUserRequest\x12\x0c\n\x04user\x18\x01 \x01(\x0c\x12\x10\n\x08password\x18\x02 \x01(\x0c\x12\x12\n\npermission\x18\x03 \x01(\r\x12\x10\n\x08\x64\x61tabase\x18\x04 \x01(\t\"\x1b\n\x0bUserRequest\x12\x0c\n\x04user\x18\x01 \x01(\x0c\"O\n\x15\x43hangePasswordRequest\x12\x0c\n\x04user\x18\x01 \x01(\x0c\x12\x13\n\x0boldPassword\x18\x02 \x01(\x0c\x12\x13\n\x0bnewPassword\x18\x03 \x01(\x0c\".\n\x0cLoginRequest\x12\x0c\n\x04user\x18\x01 \x01(\x0c\x12\x10\n\x08password\x18\x02 \x01(\x0c\"/\n\rLoginResponse\x12\r\n\x05token\x18\x01 \x01(\t\x12\x0f\n\x07warning\x18\x02 \x01(\x0c\"\x1a\n\nAuthConfig\x12\x0c\n\x04kind\x18\x01 \x01(\r\"\x1d\n\nMTLSConfig\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\"N\n\x12OpenSessionRequest\x12\x10\n\x08username\x18\x01 \x01(\x0c\x12\x10\n\x08password\x18\x02 \x01(\x0c\x12\x14\n\x0c\x64\x61tabaseName\x18\x03 \x01(\t\"<\n\x13OpenSessionResponse\x12\x11\n\tsessionID\x18\x01 \x01(\t\x12\x12\n\nserverUUID\x18\x02 \x01(\t\"\xb5\x03\n\x0cPrecondition\x12L\n\x0ckeyMustExist\x18\x01 \x01(\x0b\x32\x34.immudb.schema.Precondition.KeyMustExistPreconditionH\x00\x12R\n\x0fkeyMustNotExist\x18\x02 \x01(\x0b\x32\x37.immudb.schema.Precondition.KeyMustNotExistPreconditionH\x00\x12^\n\x15keyNotModifiedAfterTX\x18\x03 \x01(\x0b\x32=.immudb.schema.Precondition.KeyNotModifiedAfterTXPreconditionH\x00\x1a\'\n\x18KeyMustExistPrecondition\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x1a*\n\x1bKeyMustNotExistPrecondition\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x1a>\n!KeyNotModifiedAfterTXPrecondition\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\x0c\n\x04txID\x18\x02 \x01(\x04\x42\x0e\n\x0cprecondition\"S\n\x08KeyValue\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\r\n\x05value\x18\x02 \x01(\x0c\x12+\n\x08metadata\x18\x03 \x01(\x0b\x32\x19.immudb.schema.KVMetadata\"\xaf\x01\n\x05\x45ntry\x12\n\n\x02tx\x18\x01 \x01(\x04\x12\x0b\n\x03key\x18\x02 \x01(\x0c\x12\r\n\x05value\x18\x03 \x01(\x0c\x12.\n\x0creferencedBy\x18\x04 \x01(\x0b\x32\x18.immudb.schema.Reference\x12+\n\x08metadata\x18\x05 \x01(\x0b\x32\x19.immudb.schema.KVMetadata\x12\x0f\n\x07\x65xpired\x18\x06 \x01(\x08\x12\x10\n\x08revision\x18\x07 \x01(\x04\"q\n\tReference\x12\n\n\x02tx\x18\x01 \x01(\x04\x12\x0b\n\x03key\x18\x02 \x01(\x0c\x12\x0c\n\x04\x61tTx\x18\x03 \x01(\x04\x12+\n\x08metadata\x18\x04 \x01(\x0b\x32\x19.immudb.schema.KVMetadata\x12\x10\n\x08revision\x18\x05 \x01(\x04\"\x94\x01\n\x02Op\x12%\n\x02kv\x18\x01 \x01(\x0b\x32\x17.immudb.schema.KeyValueH\x00\x12*\n\x04zAdd\x18\x02 \x01(\x0b\x32\x1a.immudb.schema.ZAddRequestH\x00\x12.\n\x03ref\x18\x03 \x01(\x0b\x32\x1f.immudb.schema.ReferenceRequestH\x00\x42\x0b\n\toperation\"{\n\x0e\x45xecAllRequest\x12%\n\nOperations\x18\x01 \x03(\x0b\x32\x11.immudb.schema.Op\x12\x0e\n\x06noWait\x18\x02 \x01(\x08\x12\x32\n\rpreconditions\x18\x03 \x03(\x0b\x32\x1b.immudb.schema.Precondition\"0\n\x07\x45ntries\x12%\n\x07\x65ntries\x18\x01 \x03(\x0b\x32\x14.immudb.schema.Entry\"d\n\x06ZEntry\x12\x0b\n\x03set\x18\x01 \x01(\x0c\x12\x0b\n\x03key\x18\x02 \x01(\x0c\x12#\n\x05\x65ntry\x18\x03 \x01(\x0b\x32\x14.immudb.schema.Entry\x12\r\n\x05score\x18\x04 \x01(\x01\x12\x0c\n\x04\x61tTx\x18\x05 \x01(\x04\"2\n\x08ZEntries\x12&\n\x07\x65ntries\x18\x01 \x03(\x0b\x32\x15.immudb.schema.ZEntry\"l\n\x0bScanRequest\x12\x0f\n\x07seekKey\x18\x01 \x01(\x0c\x12\x0e\n\x06prefix\x18\x02 \x01(\x0c\x12\x0c\n\x04\x64\x65sc\x18\x03 \x01(\x08\x12\r\n\x05limit\x18\x04 \x01(\x04\x12\x0f\n\x07sinceTx\x18\x05 \x01(\x04\x12\x0e\n\x06noWait\x18\x06 \x01(\x08\"\x1b\n\tKeyPrefix\x12\x0e\n\x06prefix\x18\x01 \x01(\x0c\"\x1b\n\nEntryCount\x12\r\n\x05\x63ount\x18\x01 \x01(\x04\"1\n\tSignature\x12\x11\n\tpublicKey\x18\x01 \x01(\x0c\x12\x11\n\tsignature\x18\x02 \x01(\x0c\"\xaf\x01\n\x08TxHeader\x12\n\n\x02id\x18\x01 \x01(\x04\x12\x0f\n\x07prevAlh\x18\x02 \x01(\x0c\x12\n\n\x02ts\x18\x03 \x01(\x03\x12\x10\n\x08nentries\x18\x04 \x01(\x05\x12\n\n\x02\x65H\x18\x05 \x01(\x0c\x12\x0e\n\x06\x62lTxId\x18\x06 \x01(\x04\x12\x0e\n\x06\x62lRoot\x18\x07 \x01(\x0c\x12\x0f\n\x07version\x18\x08 \x01(\x05\x12+\n\x08metadata\x18\t \x01(\x0b\x32\x19.immudb.schema.TxMetadata\"\x0c\n\nTxMetadata\"D\n\x0bLinearProof\x12\x12\n\nsourceTxId\x18\x01 \x01(\x04\x12\x12\n\nTargetTxId\x18\x02 \x01(\x04\x12\r\n\x05terms\x18\x03 \x03(\x0c\"\x83\x02\n\tDualProof\x12/\n\x0esourceTxHeader\x18\x01 \x01(\x0b\x32\x17.immudb.schema.TxHeader\x12/\n\x0etargetTxHeader\x18\x02 \x01(\x0b\x32\x17.immudb.schema.TxHeader\x12\x16\n\x0einclusionProof\x18\x03 \x03(\x0c\x12\x18\n\x10\x63onsistencyProof\x18\x04 \x03(\x0c\x12\x15\n\rtargetBlTxAlh\x18\x05 \x01(\x0c\x12\x1a\n\x12lastInclusionProof\x18\x06 \x03(\x0c\x12/\n\x0blinearProof\x18\x07 \x01(\x0b\x32\x1a.immudb.schema.LinearProof\"\xa8\x01\n\x02Tx\x12\'\n\x06header\x18\x01 \x01(\x0b\x32\x17.immudb.schema.TxHeader\x12\'\n\x07\x65ntries\x18\x02 \x03(\x0b\x32\x16.immudb.schema.TxEntry\x12\'\n\tkvEntries\x18\x03 \x03(\x0b\x32\x14.immudb.schema.Entry\x12\'\n\x08zEntries\x18\x04 \x03(\x0b\x32\x15.immudb.schema.ZEntry\"p\n\x07TxEntry\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\x0e\n\x06hValue\x18\x02 \x01(\x0c\x12\x0c\n\x04vLen\x18\x03 \x01(\x05\x12+\n\x08metadata\x18\x04 \x01(\x0b\x32\x19.immudb.schema.KVMetadata\x12\r\n\x05value\x18\x05 \x01(\x0c\"b\n\nKVMetadata\x12\x0f\n\x07\x64\x65leted\x18\x01 \x01(\x08\x12-\n\nexpiration\x18\x02 \x01(\x0b\x32\x19.immudb.schema.Expiration\x12\x14\n\x0cnonIndexable\x18\x03 \x01(\x08\"\x1f\n\nExpiration\x12\x11\n\texpiresAt\x18\x01 \x01(\x03\"\x87\x01\n\x0cVerifiableTx\x12\x1d\n\x02tx\x18\x01 \x01(\x0b\x32\x11.immudb.schema.Tx\x12+\n\tdualProof\x18\x02 \x01(\x0b\x32\x18.immudb.schema.DualProof\x12+\n\tsignature\x18\x03 \x01(\x0b\x32\x18.immudb.schema.Signature\"\xa0\x01\n\x0fVerifiableEntry\x12#\n\x05\x65ntry\x18\x01 \x01(\x0b\x32\x14.immudb.schema.Entry\x12\x31\n\x0cverifiableTx\x18\x02 \x01(\x0b\x32\x1b.immudb.schema.VerifiableTx\x12\x35\n\x0einclusionProof\x18\x03 \x01(\x0b\x32\x1d.immudb.schema.InclusionProof\"<\n\x0eInclusionProof\x12\x0c\n\x04leaf\x18\x01 \x01(\x05\x12\r\n\x05width\x18\x02 \x01(\x05\x12\r\n\x05terms\x18\x03 \x03(\x0c\"v\n\nSetRequest\x12$\n\x03KVs\x18\x01 \x03(\x0b\x32\x17.immudb.schema.KeyValue\x12\x0e\n\x06noWait\x18\x02 \x01(\x08\x12\x32\n\rpreconditions\x18\x03 \x03(\x0b\x32\x1b.immudb.schema.Precondition\"\\\n\nKeyRequest\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\x0c\n\x04\x61tTx\x18\x02 \x01(\x04\x12\x0f\n\x07sinceTx\x18\x03 \x01(\x04\x12\x0e\n\x06noWait\x18\x04 \x01(\x08\x12\x12\n\natRevision\x18\x05 \x01(\x03\"/\n\x0eKeyListRequest\x12\x0c\n\x04keys\x18\x01 \x03(\x0c\x12\x0f\n\x07sinceTx\x18\x02 \x01(\x04\"B\n\x11\x44\x65leteKeysRequest\x12\x0c\n\x04keys\x18\x01 \x03(\x0c\x12\x0f\n\x07sinceTx\x18\x02 \x01(\x04\x12\x0e\n\x06noWait\x18\x03 \x01(\x08\"[\n\x14VerifiableSetRequest\x12-\n\nsetRequest\x18\x01 \x01(\x0b\x32\x19.immudb.schema.SetRequest\x12\x14\n\x0cproveSinceTx\x18\x02 \x01(\x04\"[\n\x14VerifiableGetRequest\x12-\n\nkeyRequest\x18\x01 \x01(\x0b\x32\x19.immudb.schema.KeyRequest\x12\x14\n\x0cproveSinceTx\x18\x02 \x01(\x04\"1\n\x0eHealthResponse\x12\x0e\n\x06status\x18\x01 \x01(\x08\x12\x0f\n\x07version\x18\x02 \x01(\t\"Q\n\x16\x44\x61tabaseHealthResponse\x12\x17\n\x0fpendingRequests\x18\x01 \x01(\r\x12\x1e\n\x16lastRequestCompletedAt\x18\x02 \x01(\x03\"g\n\x0eImmutableState\x12\n\n\x02\x64\x62\x18\x01 \x01(\t\x12\x0c\n\x04txId\x18\x02 \x01(\x04\x12\x0e\n\x06txHash\x18\x03 \x01(\x0c\x12+\n\tsignature\x18\x04 \x01(\x0b\x32\x18.immudb.schema.Signature\"\x9a\x01\n\x10ReferenceRequest\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\x15\n\rreferencedKey\x18\x02 \x01(\x0c\x12\x0c\n\x04\x61tTx\x18\x03 \x01(\x04\x12\x10\n\x08\x62oundRef\x18\x04 \x01(\x08\x12\x0e\n\x06noWait\x18\x05 \x01(\x08\x12\x32\n\rpreconditions\x18\x06 \x03(\x0b\x32\x1b.immudb.schema.Precondition\"m\n\x1aVerifiableReferenceRequest\x12\x39\n\x10referenceRequest\x18\x01 \x01(\x0b\x32\x1f.immudb.schema.ReferenceRequest\x12\x14\n\x0cproveSinceTx\x18\x02 \x01(\x04\"f\n\x0bZAddRequest\x12\x0b\n\x03set\x18\x01 \x01(\x0c\x12\r\n\x05score\x18\x02 \x01(\x01\x12\x0b\n\x03key\x18\x03 \x01(\x0c\x12\x0c\n\x04\x61tTx\x18\x04 \x01(\x04\x12\x10\n\x08\x62oundRef\x18\x05 \x01(\x08\x12\x0e\n\x06noWait\x18\x06 \x01(\x08\"\x16\n\x05Score\x12\r\n\x05score\x18\x01 \x01(\x01\"\xf6\x01\n\x0cZScanRequest\x12\x0b\n\x03set\x18\x01 \x01(\x0c\x12\x0f\n\x07seekKey\x18\x02 \x01(\x0c\x12\x11\n\tseekScore\x18\x03 \x01(\x01\x12\x10\n\x08seekAtTx\x18\x04 \x01(\x04\x12\x15\n\rinclusiveSeek\x18\x05 \x01(\x08\x12\r\n\x05limit\x18\x06 \x01(\x04\x12\x0c\n\x04\x64\x65sc\x18\x07 \x01(\x08\x12&\n\x08minScore\x18\x08 \x01(\x0b\x32\x14.immudb.schema.Score\x12&\n\x08maxScore\x18\t \x01(\x0b\x32\x14.immudb.schema.Score\x12\x0f\n\x07sinceTx\x18\n \x01(\x04\x12\x0e\n\x06noWait\x18\x0b \x01(\x08\"[\n\x0eHistoryRequest\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\x0e\n\x06offset\x18\x02 \x01(\x04\x12\r\n\x05limit\x18\x03 \x01(\x05\x12\x0c\n\x04\x64\x65sc\x18\x04 \x01(\x08\x12\x0f\n\x07sinceTx\x18\x05 \x01(\x04\"^\n\x15VerifiableZAddRequest\x12/\n\x0bzAddRequest\x18\x01 \x01(\x0b\x32\x1a.immudb.schema.ZAddRequest\x12\x14\n\x0cproveSinceTx\x18\x02 \x01(\x04\"i\n\tTxRequest\x12\n\n\x02tx\x18\x01 \x01(\x04\x12/\n\x0b\x65ntriesSpec\x18\x02 \x01(\x0b\x32\x1a.immudb.schema.EntriesSpec\x12\x0f\n\x07sinceTx\x18\x03 \x01(\x04\x12\x0e\n\x06noWait\x18\x04 \x01(\x08\"\xac\x01\n\x0b\x45ntriesSpec\x12\x33\n\rkvEntriesSpec\x18\x01 \x01(\x0b\x32\x1c.immudb.schema.EntryTypeSpec\x12\x32\n\x0czEntriesSpec\x18\x02 \x01(\x0b\x32\x1c.immudb.schema.EntryTypeSpec\x12\x34\n\x0esqlEntriesSpec\x18\x03 \x01(\x0b\x32\x1c.immudb.schema.EntryTypeSpec\"?\n\rEntryTypeSpec\x12.\n\x06\x61\x63tion\x18\x01 \x01(\x0e\x32\x1e.immudb.schema.EntryTypeAction\"\x89\x01\n\x13VerifiableTxRequest\x12\n\n\x02tx\x18\x01 \x01(\x04\x12\x14\n\x0cproveSinceTx\x18\x02 \x01(\x04\x12/\n\x0b\x65ntriesSpec\x18\x03 \x01(\x0b\x32\x1a.immudb.schema.EntriesSpec\x12\x0f\n\x07sinceTx\x18\x04 \x01(\x04\x12\x0e\n\x06noWait\x18\x05 \x01(\x08\"\x91\x01\n\rTxScanRequest\x12\x11\n\tinitialTx\x18\x01 \x01(\x04\x12\r\n\x05limit\x18\x02 \x01(\r\x12\x0c\n\x04\x64\x65sc\x18\x03 \x01(\x08\x12/\n\x0b\x65ntriesSpec\x18\x04 \x01(\x0b\x32\x1a.immudb.schema.EntriesSpec\x12\x0f\n\x07sinceTx\x18\x05 \x01(\x04\x12\x0e\n\x06noWait\x18\x06 \x01(\x08\"(\n\x06TxList\x12\x1e\n\x03txs\x18\x01 \x03(\x0b\x32\x11.immudb.schema.Tx\"\x1d\n\x0f\x45xportTxRequest\x12\n\n\x02tx\x18\x01 \x01(\x04\" \n\x08\x44\x61tabase\x12\x14\n\x0c\x64\x61tabaseName\x18\x01 \x01(\t\"\x9b\x02\n\x10\x44\x61tabaseSettings\x12\x14\n\x0c\x64\x61tabaseName\x18\x01 \x01(\t\x12\x0f\n\x07replica\x18\x02 \x01(\x08\x12\x16\n\x0emasterDatabase\x18\x03 \x01(\t\x12\x15\n\rmasterAddress\x18\x04 \x01(\t\x12\x12\n\nmasterPort\x18\x05 \x01(\r\x12\x18\n\x10\x66ollowerUsername\x18\x06 \x01(\t\x12\x18\n\x10\x66ollowerPassword\x18\x07 \x01(\t\x12\x10\n\x08\x66ileSize\x18\x08 \x01(\r\x12\x11\n\tmaxKeyLen\x18\t \x01(\r\x12\x13\n\x0bmaxValueLen\x18\n \x01(\r\x12\x14\n\x0cmaxTxEntries\x18\x0b \x01(\r\x12\x19\n\x11\x65xcludeCommitTime\x18\x0c \x01(\x08\"`\n\x15\x43reateDatabaseRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x39\n\x08settings\x18\x02 \x01(\x0b\x32\'.immudb.schema.DatabaseNullableSettings\"a\n\x16\x43reateDatabaseResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x39\n\x08settings\x18\x02 \x01(\x0b\x32\'.immudb.schema.DatabaseNullableSettings\"d\n\x15UpdateDatabaseRequest\x12\x10\n\x08\x64\x61tabase\x18\x01 \x01(\t\x12\x39\n\x08settings\x18\x02 \x01(\x0b\x32\'.immudb.schema.DatabaseNullableSettings\"e\n\x16UpdateDatabaseResponse\x12\x10\n\x08\x64\x61tabase\x18\x01 \x01(\t\x12\x39\n\x08settings\x18\x02 \x01(\x0b\x32\'.immudb.schema.DatabaseNullableSettings\"\x19\n\x17\x44\x61tabaseSettingsRequest\"g\n\x18\x44\x61tabaseSettingsResponse\x12\x10\n\x08\x64\x61tabase\x18\x01 \x01(\t\x12\x39\n\x08settings\x18\x02 \x01(\x0b\x32\'.immudb.schema.DatabaseNullableSettings\"\x1f\n\x0eNullableUint32\x12\r\n\x05value\x18\x01 \x01(\r\"\x1f\n\x0eNullableUint64\x12\r\n\x05value\x18\x01 \x01(\x04\"\x1e\n\rNullableFloat\x12\r\n\x05value\x18\x01 \x01(\x02\"\x1d\n\x0cNullableBool\x12\r\n\x05value\x18\x01 \x01(\x08\"\x1f\n\x0eNullableString\x12\r\n\x05value\x18\x01 \x01(\t\"\xee\x06\n\x18\x44\x61tabaseNullableSettings\x12G\n\x13replicationSettings\x18\x02 \x01(\x0b\x32*.immudb.schema.ReplicationNullableSettings\x12/\n\x08\x66ileSize\x18\x08 \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12\x30\n\tmaxKeyLen\x18\t \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12\x32\n\x0bmaxValueLen\x18\n \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12\x33\n\x0cmaxTxEntries\x18\x0b \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12\x36\n\x11\x65xcludeCommitTime\x18\x0c \x01(\x0b\x32\x1b.immudb.schema.NullableBool\x12\x35\n\x0emaxConcurrency\x18\r \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12\x37\n\x10maxIOConcurrency\x18\x0e \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12\x35\n\x0etxLogCacheSize\x18\x0f \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12\x39\n\x12vLogMaxOpenedFiles\x18\x10 \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12:\n\x13txLogMaxOpenedFiles\x18\x11 \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12>\n\x17\x63ommitLogMaxOpenedFiles\x18\x12 \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12;\n\rindexSettings\x18\x13 \x01(\x0b\x32$.immudb.schema.IndexNullableSettings\x12;\n\x14writeTxHeaderVersion\x18\x14 \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12-\n\x08\x61utoload\x18\x15 \x01(\x0b\x32\x1b.immudb.schema.NullableBool\"\xdd\x02\n\x1bReplicationNullableSettings\x12,\n\x07replica\x18\x01 \x01(\x0b\x32\x1b.immudb.schema.NullableBool\x12\x35\n\x0emasterDatabase\x18\x02 \x01(\x0b\x32\x1d.immudb.schema.NullableString\x12\x34\n\rmasterAddress\x18\x03 \x01(\x0b\x32\x1d.immudb.schema.NullableString\x12\x31\n\nmasterPort\x18\x04 \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12\x37\n\x10\x66ollowerUsername\x18\x05 \x01(\x0b\x32\x1d.immudb.schema.NullableString\x12\x37\n\x10\x66ollowerPassword\x18\x06 \x01(\x0b\x32\x1d.immudb.schema.NullableString\"\x86\x06\n\x15IndexNullableSettings\x12\x35\n\x0e\x66lushThreshold\x18\x01 \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12\x34\n\rsyncThreshold\x18\x02 \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12\x30\n\tcacheSize\x18\x03 \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12\x32\n\x0bmaxNodeSize\x18\x04 \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12\x39\n\x12maxActiveSnapshots\x18\x05 \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12\x39\n\x12renewSnapRootAfter\x18\x06 \x01(\x0b\x32\x1d.immudb.schema.NullableUint64\x12\x35\n\x0e\x63ompactionThld\x18\x07 \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12<\n\x15\x64\x65layDuringCompaction\x18\x08 \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12=\n\x16nodesLogMaxOpenedFiles\x18\t \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12?\n\x18historyLogMaxOpenedFiles\x18\n \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12>\n\x17\x63ommitLogMaxOpenedFiles\x18\x0b \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12\x36\n\x0f\x66lushBufferSize\x18\x0c \x01(\x0b\x32\x1d.immudb.schema.NullableUint32\x12\x37\n\x11\x63leanupPercentage\x18\r \x01(\x0b\x32\x1c.immudb.schema.NullableFloat\"\'\n\x13LoadDatabaseRequest\x12\x10\n\x08\x64\x61tabase\x18\x01 \x01(\t\"(\n\x14LoadDatabaseResponse\x12\x10\n\x08\x64\x61tabase\x18\x01 \x01(\t\")\n\x15UnloadDatabaseRequest\x12\x10\n\x08\x64\x61tabase\x18\x01 \x01(\t\"*\n\x16UnloadDatabaseResponse\x12\x10\n\x08\x64\x61tabase\x18\x01 \x01(\t\")\n\x15\x44\x65leteDatabaseRequest\x12\x10\n\x08\x64\x61tabase\x18\x01 \x01(\t\"*\n\x16\x44\x65leteDatabaseResponse\x12\x10\n\x08\x64\x61tabase\x18\x01 \x01(\t\">\n\x11\x46lushIndexRequest\x12\x19\n\x11\x63leanupPercentage\x18\x01 \x01(\x02\x12\x0e\n\x06synced\x18\x02 \x01(\x08\"&\n\x12\x46lushIndexResponse\x12\x10\n\x08\x64\x61tabase\x18\x01 \x01(\t\"\x1a\n\x05Table\x12\x11\n\ttableName\x18\x01 \x01(\t\"h\n\rSQLGetRequest\x12\r\n\x05table\x18\x01 \x01(\t\x12)\n\x08pkValues\x18\x02 \x03(\x0b\x32\x17.immudb.schema.SQLValue\x12\x0c\n\x04\x61tTx\x18\x03 \x01(\x04\x12\x0f\n\x07sinceTx\x18\x04 \x01(\x04\"d\n\x17VerifiableSQLGetRequest\x12\x33\n\rsqlGetRequest\x18\x01 \x01(\x0b\x32\x1c.immudb.schema.SQLGetRequest\x12\x14\n\x0cproveSinceTx\x18\x02 \x01(\x04\"_\n\x08SQLEntry\x12\n\n\x02tx\x18\x01 \x01(\x04\x12\x0b\n\x03key\x18\x02 \x01(\x0c\x12\r\n\x05value\x18\x03 \x01(\x0c\x12+\n\x08metadata\x18\x04 \x01(\x0b\x32\x19.immudb.schema.KVMetadata\"\xdd\x05\n\x12VerifiableSQLEntry\x12)\n\x08sqlEntry\x18\x01 \x01(\x0b\x32\x17.immudb.schema.SQLEntry\x12\x31\n\x0cverifiableTx\x18\x02 \x01(\x0b\x32\x1b.immudb.schema.VerifiableTx\x12\x35\n\x0einclusionProof\x18\x03 \x01(\x0b\x32\x1d.immudb.schema.InclusionProof\x12\x12\n\nDatabaseId\x18\x04 \x01(\r\x12\x0f\n\x07TableId\x18\x05 \x01(\r\x12\r\n\x05PKIDs\x18\x10 \x03(\r\x12I\n\x0c\x43olNamesById\x18\x08 \x03(\x0b\x32\x33.immudb.schema.VerifiableSQLEntry.ColNamesByIdEntry\x12I\n\x0c\x43olIdsByName\x18\t \x03(\x0b\x32\x33.immudb.schema.VerifiableSQLEntry.ColIdsByNameEntry\x12I\n\x0c\x43olTypesById\x18\n \x03(\x0b\x32\x33.immudb.schema.VerifiableSQLEntry.ColTypesByIdEntry\x12\x45\n\nColLenById\x18\x0b \x03(\x0b\x32\x31.immudb.schema.VerifiableSQLEntry.ColLenByIdEntry\x1a\x33\n\x11\x43olNamesByIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x33\n\x11\x43olIdsByNameEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\r:\x02\x38\x01\x1a\x33\n\x11\x43olTypesByIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x31\n\x0f\x43olLenByIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01J\x04\x08\x06\x10\x07\"!\n\x10UseDatabaseReply\x12\r\n\x05token\x18\x01 \x01(\t\"\x82\x01\n\x17\x43hangePermissionRequest\x12/\n\x06\x61\x63tion\x18\x01 \x01(\x0e\x32\x1f.immudb.schema.PermissionAction\x12\x10\n\x08username\x18\x02 \x01(\t\x12\x10\n\x08\x64\x61tabase\x18\x03 \x01(\t\x12\x12\n\npermission\x18\x04 \x01(\r\"8\n\x14SetActiveUserRequest\x12\x0e\n\x06\x61\x63tive\x18\x01 \x01(\x08\x12\x10\n\x08username\x18\x02 \x01(\t\"B\n\x14\x44\x61tabaseListResponse\x12*\n\tdatabases\x18\x01 \x03(\x0b\x32\x17.immudb.schema.Database\"\x17\n\x15\x44\x61tabaseListRequestV2\"P\n\x16\x44\x61tabaseListResponseV2\x12\x36\n\tdatabases\x18\x01 \x03(\x0b\x32#.immudb.schema.DatabaseWithSettings\"o\n\x14\x44\x61tabaseWithSettings\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x39\n\x08settings\x18\x02 \x01(\x0b\x32\'.immudb.schema.DatabaseNullableSettings\x12\x0e\n\x06loaded\x18\x03 \x01(\x08\"\x18\n\x05\x43hunk\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c\"9\n\x12UseSnapshotRequest\x12\x0f\n\x07sinceTx\x18\x01 \x01(\x04\x12\x12\n\nasBeforeTx\x18\x02 \x01(\x04\"X\n\x0eSQLExecRequest\x12\x0b\n\x03sql\x18\x01 \x01(\t\x12)\n\x06params\x18\x02 \x03(\x0b\x32\x19.immudb.schema.NamedParam\x12\x0e\n\x06noWait\x18\x03 \x01(\x08\"`\n\x0fSQLQueryRequest\x12\x0b\n\x03sql\x18\x01 \x01(\t\x12)\n\x06params\x18\x02 \x03(\x0b\x32\x19.immudb.schema.NamedParam\x12\x15\n\rreuseSnapshot\x18\x03 \x01(\x08\"B\n\nNamedParam\x12\x0c\n\x04name\x18\x01 \x01(\t\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.immudb.schema.SQLValue\"N\n\rSQLExecResult\x12*\n\x03txs\x18\x05 \x03(\x0b\x32\x1d.immudb.schema.CommittedSQLTx\x12\x11\n\tongoingTx\x18\x06 \x01(\x08\"\x8d\x03\n\x0e\x43ommittedSQLTx\x12\'\n\x06header\x18\x01 \x01(\x0b\x32\x17.immudb.schema.TxHeader\x12\x13\n\x0bupdatedRows\x18\x02 \x01(\r\x12K\n\x0flastInsertedPKs\x18\x03 \x03(\x0b\x32\x32.immudb.schema.CommittedSQLTx.LastInsertedPKsEntry\x12M\n\x10\x66irstInsertedPKs\x18\x04 \x03(\x0b\x32\x33.immudb.schema.CommittedSQLTx.FirstInsertedPKsEntry\x1aO\n\x14LastInsertedPKsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.immudb.schema.SQLValue:\x02\x38\x01\x1aP\n\x15\x46irstInsertedPKsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.immudb.schema.SQLValue:\x02\x38\x01\"Z\n\x0eSQLQueryResult\x12&\n\x07\x63olumns\x18\x02 \x03(\x0b\x32\x15.immudb.schema.Column\x12 \n\x04rows\x18\x01 \x03(\x0b\x32\x12.immudb.schema.Row\"$\n\x06\x43olumn\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\"?\n\x03Row\x12\x0f\n\x07\x63olumns\x18\x01 \x03(\t\x12\'\n\x06values\x18\x02 \x03(\x0b\x32\x17.immudb.schema.SQLValue\"\x82\x01\n\x08SQLValue\x12*\n\x04null\x18\x01 \x01(\x0e\x32\x1a.google.protobuf.NullValueH\x00\x12\x0b\n\x01n\x18\x02 \x01(\x03H\x00\x12\x0b\n\x01s\x18\x03 \x01(\tH\x00\x12\x0b\n\x01\x62\x18\x04 \x01(\x08H\x00\x12\x0c\n\x02\x62s\x18\x05 \x01(\x0cH\x00\x12\x0c\n\x02ts\x18\x06 \x01(\x03H\x00\x42\x07\n\x05value\"3\n\x0cNewTxRequest\x12#\n\x04mode\x18\x01 \x01(\x0e\x32\x15.immudb.schema.TxMode\"&\n\rNewTxResponse\x12\x15\n\rtransactionID\x18\x01 \x01(\t\"(\n\tErrorInfo\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\x12\r\n\x05\x63\x61use\x18\x02 \x01(\t\"\x1a\n\tDebugInfo\x12\r\n\x05stack\x18\x01 \x01(\t\" \n\tRetryInfo\x12\x13\n\x0bretry_delay\x18\x01 \x01(\x05*K\n\x0f\x45ntryTypeAction\x12\x0b\n\x07\x45XCLUDE\x10\x00\x12\x0f\n\x0bONLY_DIGEST\x10\x01\x12\r\n\tRAW_VALUE\x10\x02\x12\x0b\n\x07RESOLVE\x10\x03*)\n\x10PermissionAction\x12\t\n\x05GRANT\x10\x00\x12\n\n\x06REVOKE\x10\x01*4\n\x06TxMode\x12\x0c\n\x08ReadOnly\x10\x00\x12\r\n\tWriteOnly\x10\x01\x12\r\n\tReadWrite\x10\x02\x32\x82(\n\x0bImmuService\x12<\n\tListUsers\x12\x16.google.protobuf.Empty\x1a\x17.immudb.schema.UserList\x12\x46\n\nCreateUser\x12 .immudb.schema.CreateUserRequest\x1a\x16.google.protobuf.Empty\x12N\n\x0e\x43hangePassword\x12$.immudb.schema.ChangePasswordRequest\x1a\x16.google.protobuf.Empty\x12R\n\x10\x43hangePermission\x12&.immudb.schema.ChangePermissionRequest\x1a\x16.google.protobuf.Empty\x12L\n\rSetActiveUser\x12#.immudb.schema.SetActiveUserRequest\x1a\x16.google.protobuf.Empty\x12\x45\n\x10UpdateAuthConfig\x12\x19.immudb.schema.AuthConfig\x1a\x16.google.protobuf.Empty\x12\x45\n\x10UpdateMTLSConfig\x12\x19.immudb.schema.MTLSConfig\x1a\x16.google.protobuf.Empty\x12V\n\x0bOpenSession\x12!.immudb.schema.OpenSessionRequest\x1a\".immudb.schema.OpenSessionResponse\"\x00\x12@\n\x0c\x43loseSession\x12\x16.google.protobuf.Empty\x1a\x16.google.protobuf.Empty\"\x00\x12=\n\tKeepAlive\x12\x16.google.protobuf.Empty\x1a\x16.google.protobuf.Empty\"\x00\x12\x44\n\x05NewTx\x12\x1b.immudb.schema.NewTxRequest\x1a\x1c.immudb.schema.NewTxResponse\"\x00\x12\x41\n\x06\x43ommit\x12\x16.google.protobuf.Empty\x1a\x1d.immudb.schema.CommittedSQLTx\"\x00\x12<\n\x08Rollback\x12\x16.google.protobuf.Empty\x1a\x16.google.protobuf.Empty\"\x00\x12\x44\n\tTxSQLExec\x12\x1d.immudb.schema.SQLExecRequest\x1a\x16.google.protobuf.Empty\"\x00\x12M\n\nTxSQLQuery\x12\x1e.immudb.schema.SQLQueryRequest\x1a\x1d.immudb.schema.SQLQueryResult\"\x00\x12\x42\n\x05Login\x12\x1b.immudb.schema.LoginRequest\x1a\x1c.immudb.schema.LoginResponse\x12\x38\n\x06Logout\x12\x16.google.protobuf.Empty\x1a\x16.google.protobuf.Empty\x12\x39\n\x03Set\x12\x19.immudb.schema.SetRequest\x1a\x17.immudb.schema.TxHeader\x12Q\n\rVerifiableSet\x12#.immudb.schema.VerifiableSetRequest\x1a\x1b.immudb.schema.VerifiableTx\x12\x36\n\x03Get\x12\x19.immudb.schema.KeyRequest\x1a\x14.immudb.schema.Entry\x12T\n\rVerifiableGet\x12#.immudb.schema.VerifiableGetRequest\x1a\x1e.immudb.schema.VerifiableEntry\x12\x43\n\x06\x44\x65lete\x12 .immudb.schema.DeleteKeysRequest\x1a\x17.immudb.schema.TxHeader\x12?\n\x06GetAll\x12\x1d.immudb.schema.KeyListRequest\x1a\x16.immudb.schema.Entries\x12\x41\n\x07\x45xecAll\x12\x1d.immudb.schema.ExecAllRequest\x1a\x17.immudb.schema.TxHeader\x12:\n\x04Scan\x12\x1a.immudb.schema.ScanRequest\x1a\x16.immudb.schema.Entries\x12<\n\x05\x43ount\x12\x18.immudb.schema.KeyPrefix\x1a\x19.immudb.schema.EntryCount\x12=\n\x08\x43ountAll\x12\x16.google.protobuf.Empty\x1a\x19.immudb.schema.EntryCount\x12\x35\n\x06TxById\x12\x18.immudb.schema.TxRequest\x1a\x11.immudb.schema.Tx\x12S\n\x10VerifiableTxById\x12\".immudb.schema.VerifiableTxRequest\x1a\x1b.immudb.schema.VerifiableTx\x12=\n\x06TxScan\x12\x1c.immudb.schema.TxScanRequest\x1a\x15.immudb.schema.TxList\x12@\n\x07History\x12\x1d.immudb.schema.HistoryRequest\x1a\x16.immudb.schema.Entries\x12?\n\x06Health\x12\x16.google.protobuf.Empty\x1a\x1d.immudb.schema.HealthResponse\x12O\n\x0e\x44\x61tabaseHealth\x12\x16.google.protobuf.Empty\x1a%.immudb.schema.DatabaseHealthResponse\x12\x45\n\x0c\x43urrentState\x12\x16.google.protobuf.Empty\x1a\x1d.immudb.schema.ImmutableState\x12H\n\x0cSetReference\x12\x1f.immudb.schema.ReferenceRequest\x1a\x17.immudb.schema.TxHeader\x12`\n\x16VerifiableSetReference\x12).immudb.schema.VerifiableReferenceRequest\x1a\x1b.immudb.schema.VerifiableTx\x12;\n\x04ZAdd\x12\x1a.immudb.schema.ZAddRequest\x1a\x17.immudb.schema.TxHeader\x12S\n\x0eVerifiableZAdd\x12$.immudb.schema.VerifiableZAddRequest\x1a\x1b.immudb.schema.VerifiableTx\x12=\n\x05ZScan\x12\x1b.immudb.schema.ZScanRequest\x1a\x17.immudb.schema.ZEntries\x12\x41\n\x0e\x43reateDatabase\x12\x17.immudb.schema.Database\x1a\x16.google.protobuf.Empty\x12M\n\x12\x43reateDatabaseWith\x12\x1f.immudb.schema.DatabaseSettings\x1a\x16.google.protobuf.Empty\x12_\n\x10\x43reateDatabaseV2\x12$.immudb.schema.CreateDatabaseRequest\x1a%.immudb.schema.CreateDatabaseResponse\x12W\n\x0cLoadDatabase\x12\".immudb.schema.LoadDatabaseRequest\x1a#.immudb.schema.LoadDatabaseResponse\x12]\n\x0eUnloadDatabase\x12$.immudb.schema.UnloadDatabaseRequest\x1a%.immudb.schema.UnloadDatabaseResponse\x12]\n\x0e\x44\x65leteDatabase\x12$.immudb.schema.DeleteDatabaseRequest\x1a%.immudb.schema.DeleteDatabaseResponse\x12K\n\x0c\x44\x61tabaseList\x12\x16.google.protobuf.Empty\x1a#.immudb.schema.DatabaseListResponse\x12]\n\x0e\x44\x61tabaseListV2\x12$.immudb.schema.DatabaseListRequestV2\x1a%.immudb.schema.DatabaseListResponseV2\x12G\n\x0bUseDatabase\x12\x17.immudb.schema.Database\x1a\x1f.immudb.schema.UseDatabaseReply\x12I\n\x0eUpdateDatabase\x12\x1f.immudb.schema.DatabaseSettings\x1a\x16.google.protobuf.Empty\x12_\n\x10UpdateDatabaseV2\x12$.immudb.schema.UpdateDatabaseRequest\x1a%.immudb.schema.UpdateDatabaseResponse\x12N\n\x13GetDatabaseSettings\x12\x16.google.protobuf.Empty\x1a\x1f.immudb.schema.DatabaseSettings\x12h\n\x15GetDatabaseSettingsV2\x12&.immudb.schema.DatabaseSettingsRequest\x1a\'.immudb.schema.DatabaseSettingsResponse\x12Q\n\nFlushIndex\x12 .immudb.schema.FlushIndexRequest\x1a!.immudb.schema.FlushIndexResponse\x12>\n\x0c\x43ompactIndex\x12\x16.google.protobuf.Empty\x1a\x16.google.protobuf.Empty\x12@\n\tstreamGet\x12\x19.immudb.schema.KeyRequest\x1a\x14.immudb.schema.Chunk\"\x00\x30\x01\x12>\n\tstreamSet\x12\x14.immudb.schema.Chunk\x1a\x17.immudb.schema.TxHeader\"\x00(\x01\x12T\n\x13streamVerifiableGet\x12#.immudb.schema.VerifiableGetRequest\x1a\x14.immudb.schema.Chunk\"\x00\x30\x01\x12L\n\x13streamVerifiableSet\x12\x14.immudb.schema.Chunk\x1a\x1b.immudb.schema.VerifiableTx\"\x00(\x01\x12\x42\n\nstreamScan\x12\x1a.immudb.schema.ScanRequest\x1a\x14.immudb.schema.Chunk\"\x00\x30\x01\x12\x44\n\x0bstreamZScan\x12\x1b.immudb.schema.ZScanRequest\x1a\x14.immudb.schema.Chunk\"\x00\x30\x01\x12H\n\rstreamHistory\x12\x1d.immudb.schema.HistoryRequest\x1a\x14.immudb.schema.Chunk\"\x00\x30\x01\x12\x42\n\rstreamExecAll\x12\x14.immudb.schema.Chunk\x1a\x17.immudb.schema.TxHeader\"\x00(\x01\x12\x44\n\x08\x65xportTx\x12\x1e.immudb.schema.ExportTxRequest\x1a\x14.immudb.schema.Chunk\"\x00\x30\x01\x12@\n\x0breplicateTx\x12\x14.immudb.schema.Chunk\x1a\x17.immudb.schema.TxHeader\"\x00(\x01\x12\x46\n\x07SQLExec\x12\x1d.immudb.schema.SQLExecRequest\x1a\x1c.immudb.schema.SQLExecResult\x12I\n\x08SQLQuery\x12\x1e.immudb.schema.SQLQueryRequest\x1a\x1d.immudb.schema.SQLQueryResult\x12\x43\n\nListTables\x12\x16.google.protobuf.Empty\x1a\x1d.immudb.schema.SQLQueryResult\x12\x44\n\rDescribeTable\x12\x14.immudb.schema.Table\x1a\x1d.immudb.schema.SQLQueryResult\x12]\n\x10VerifiableSQLGet\x12&.immudb.schema.VerifiableSQLGetRequest\x1a!.immudb.schema.VerifiableSQLEntryB-Z+github.com/codenotary/immudb/pkg/api/schemab\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,google_dot_protobuf_dot_struct__pb2.DESCRIPTOR,])

_ENTRYTYPEACTION = _descriptor.EnumDescriptor(
  name='EntryTypeAction',
  full_name='immudb.schema.EntryTypeAction',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='EXCLUDE', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ONLY_DIGEST', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RAW_VALUE', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RESOLVE', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=12269,
  serialized_end=12344,
)
_sym_db.RegisterEnumDescriptor(_ENTRYTYPEACTION)

EntryTypeAction = enum_type_wrapper.EnumTypeWrapper(_ENTRYTYPEACTION)
_PERMISSIONACTION = _descriptor.EnumDescriptor(
  name='PermissionAction',
  full_name='immudb.schema.PermissionAction',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='GRANT', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='REVOKE', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=12346,
  serialized_end=12387,
)
_sym_db.RegisterEnumDescriptor(_PERMISSIONACTION)

PermissionAction = enum_type_wrapper.EnumTypeWrapper(_PERMISSIONACTION)
_TXMODE = _descriptor.EnumDescriptor(
  name='TxMode',
  full_name='immudb.schema.TxMode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ReadOnly', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WriteOnly', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ReadWrite', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=12389,
  serialized_end=12441,
)
_sym_db.RegisterEnumDescriptor(_TXMODE)

TxMode = enum_type_wrapper.EnumTypeWrapper(_TXMODE)
EXCLUDE = 0
ONLY_DIGEST = 1
RAW_VALUE = 2
RESOLVE = 3
GRANT = 0
REVOKE = 1
ReadOnly = 0
WriteOnly = 1
ReadWrite = 2



_KEY = _descriptor.Descriptor(
  name='Key',
  full_name='immudb.schema.Key',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.Key.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=90,
  serialized_end=108,
)


_PERMISSION = _descriptor.Descriptor(
  name='Permission',
  full_name='immudb.schema.Permission',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='database', full_name='immudb.schema.Permission.database', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='permission', full_name='immudb.schema.Permission.permission', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=110,
  serialized_end=160,
)


_USER = _descriptor.Descriptor(
  name='User',
  full_name='immudb.schema.User',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user', full_name='immudb.schema.User.user', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='permissions', full_name='immudb.schema.User.permissions', index=1,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='createdby', full_name='immudb.schema.User.createdby', index=2,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='createdat', full_name='immudb.schema.User.createdat', index=3,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='active', full_name='immudb.schema.User.active', index=4,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=162,
  serialized_end=284,
)


_USERLIST = _descriptor.Descriptor(
  name='UserList',
  full_name='immudb.schema.UserList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='users', full_name='immudb.schema.UserList.users', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=286,
  serialized_end=332,
)


_CREATEUSERREQUEST = _descriptor.Descriptor(
  name='CreateUserRequest',
  full_name='immudb.schema.CreateUserRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user', full_name='immudb.schema.CreateUserRequest.user', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='password', full_name='immudb.schema.CreateUserRequest.password', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='permission', full_name='immudb.schema.CreateUserRequest.permission', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='database', full_name='immudb.schema.CreateUserRequest.database', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=334,
  serialized_end=423,
)


_USERREQUEST = _descriptor.Descriptor(
  name='UserRequest',
  full_name='immudb.schema.UserRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user', full_name='immudb.schema.UserRequest.user', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=425,
  serialized_end=452,
)


_CHANGEPASSWORDREQUEST = _descriptor.Descriptor(
  name='ChangePasswordRequest',
  full_name='immudb.schema.ChangePasswordRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user', full_name='immudb.schema.ChangePasswordRequest.user', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='oldPassword', full_name='immudb.schema.ChangePasswordRequest.oldPassword', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='newPassword', full_name='immudb.schema.ChangePasswordRequest.newPassword', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=454,
  serialized_end=533,
)


_LOGINREQUEST = _descriptor.Descriptor(
  name='LoginRequest',
  full_name='immudb.schema.LoginRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user', full_name='immudb.schema.LoginRequest.user', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='password', full_name='immudb.schema.LoginRequest.password', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=535,
  serialized_end=581,
)


_LOGINRESPONSE = _descriptor.Descriptor(
  name='LoginResponse',
  full_name='immudb.schema.LoginResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='token', full_name='immudb.schema.LoginResponse.token', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='warning', full_name='immudb.schema.LoginResponse.warning', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=583,
  serialized_end=630,
)


_AUTHCONFIG = _descriptor.Descriptor(
  name='AuthConfig',
  full_name='immudb.schema.AuthConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='kind', full_name='immudb.schema.AuthConfig.kind', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=632,
  serialized_end=658,
)


_MTLSCONFIG = _descriptor.Descriptor(
  name='MTLSConfig',
  full_name='immudb.schema.MTLSConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='enabled', full_name='immudb.schema.MTLSConfig.enabled', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=660,
  serialized_end=689,
)


_OPENSESSIONREQUEST = _descriptor.Descriptor(
  name='OpenSessionRequest',
  full_name='immudb.schema.OpenSessionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='username', full_name='immudb.schema.OpenSessionRequest.username', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='password', full_name='immudb.schema.OpenSessionRequest.password', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='databaseName', full_name='immudb.schema.OpenSessionRequest.databaseName', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=691,
  serialized_end=769,
)


_OPENSESSIONRESPONSE = _descriptor.Descriptor(
  name='OpenSessionResponse',
  full_name='immudb.schema.OpenSessionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sessionID', full_name='immudb.schema.OpenSessionResponse.sessionID', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='serverUUID', full_name='immudb.schema.OpenSessionResponse.serverUUID', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=771,
  serialized_end=831,
)


_PRECONDITION_KEYMUSTEXISTPRECONDITION = _descriptor.Descriptor(
  name='KeyMustExistPrecondition',
  full_name='immudb.schema.Precondition.KeyMustExistPrecondition',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.Precondition.KeyMustExistPrecondition.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1108,
  serialized_end=1147,
)

_PRECONDITION_KEYMUSTNOTEXISTPRECONDITION = _descriptor.Descriptor(
  name='KeyMustNotExistPrecondition',
  full_name='immudb.schema.Precondition.KeyMustNotExistPrecondition',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.Precondition.KeyMustNotExistPrecondition.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1149,
  serialized_end=1191,
)

_PRECONDITION_KEYNOTMODIFIEDAFTERTXPRECONDITION = _descriptor.Descriptor(
  name='KeyNotModifiedAfterTXPrecondition',
  full_name='immudb.schema.Precondition.KeyNotModifiedAfterTXPrecondition',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.Precondition.KeyNotModifiedAfterTXPrecondition.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='txID', full_name='immudb.schema.Precondition.KeyNotModifiedAfterTXPrecondition.txID', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1193,
  serialized_end=1255,
)

_PRECONDITION = _descriptor.Descriptor(
  name='Precondition',
  full_name='immudb.schema.Precondition',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='keyMustExist', full_name='immudb.schema.Precondition.keyMustExist', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='keyMustNotExist', full_name='immudb.schema.Precondition.keyMustNotExist', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='keyNotModifiedAfterTX', full_name='immudb.schema.Precondition.keyNotModifiedAfterTX', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_PRECONDITION_KEYMUSTEXISTPRECONDITION, _PRECONDITION_KEYMUSTNOTEXISTPRECONDITION, _PRECONDITION_KEYNOTMODIFIEDAFTERTXPRECONDITION, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='precondition', full_name='immudb.schema.Precondition.precondition',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=834,
  serialized_end=1271,
)


_KEYVALUE = _descriptor.Descriptor(
  name='KeyValue',
  full_name='immudb.schema.KeyValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.KeyValue.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='immudb.schema.KeyValue.value', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='immudb.schema.KeyValue.metadata', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1273,
  serialized_end=1356,
)


_ENTRY = _descriptor.Descriptor(
  name='Entry',
  full_name='immudb.schema.Entry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tx', full_name='immudb.schema.Entry.tx', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.Entry.key', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='immudb.schema.Entry.value', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='referencedBy', full_name='immudb.schema.Entry.referencedBy', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='immudb.schema.Entry.metadata', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='expired', full_name='immudb.schema.Entry.expired', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='revision', full_name='immudb.schema.Entry.revision', index=6,
      number=7, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1359,
  serialized_end=1534,
)


_REFERENCE = _descriptor.Descriptor(
  name='Reference',
  full_name='immudb.schema.Reference',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tx', full_name='immudb.schema.Reference.tx', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.Reference.key', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='atTx', full_name='immudb.schema.Reference.atTx', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='immudb.schema.Reference.metadata', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='revision', full_name='immudb.schema.Reference.revision', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1536,
  serialized_end=1649,
)


_OP = _descriptor.Descriptor(
  name='Op',
  full_name='immudb.schema.Op',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='kv', full_name='immudb.schema.Op.kv', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='zAdd', full_name='immudb.schema.Op.zAdd', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ref', full_name='immudb.schema.Op.ref', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='operation', full_name='immudb.schema.Op.operation',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=1652,
  serialized_end=1800,
)


_EXECALLREQUEST = _descriptor.Descriptor(
  name='ExecAllRequest',
  full_name='immudb.schema.ExecAllRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='Operations', full_name='immudb.schema.ExecAllRequest.Operations', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='noWait', full_name='immudb.schema.ExecAllRequest.noWait', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='preconditions', full_name='immudb.schema.ExecAllRequest.preconditions', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1802,
  serialized_end=1925,
)


_ENTRIES = _descriptor.Descriptor(
  name='Entries',
  full_name='immudb.schema.Entries',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='entries', full_name='immudb.schema.Entries.entries', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1927,
  serialized_end=1975,
)


_ZENTRY = _descriptor.Descriptor(
  name='ZEntry',
  full_name='immudb.schema.ZEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='set', full_name='immudb.schema.ZEntry.set', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.ZEntry.key', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='entry', full_name='immudb.schema.ZEntry.entry', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='score', full_name='immudb.schema.ZEntry.score', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='atTx', full_name='immudb.schema.ZEntry.atTx', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1977,
  serialized_end=2077,
)


_ZENTRIES = _descriptor.Descriptor(
  name='ZEntries',
  full_name='immudb.schema.ZEntries',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='entries', full_name='immudb.schema.ZEntries.entries', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2079,
  serialized_end=2129,
)


_SCANREQUEST = _descriptor.Descriptor(
  name='ScanRequest',
  full_name='immudb.schema.ScanRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='seekKey', full_name='immudb.schema.ScanRequest.seekKey', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='prefix', full_name='immudb.schema.ScanRequest.prefix', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='desc', full_name='immudb.schema.ScanRequest.desc', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='limit', full_name='immudb.schema.ScanRequest.limit', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sinceTx', full_name='immudb.schema.ScanRequest.sinceTx', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='noWait', full_name='immudb.schema.ScanRequest.noWait', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2131,
  serialized_end=2239,
)


_KEYPREFIX = _descriptor.Descriptor(
  name='KeyPrefix',
  full_name='immudb.schema.KeyPrefix',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='prefix', full_name='immudb.schema.KeyPrefix.prefix', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2241,
  serialized_end=2268,
)


_ENTRYCOUNT = _descriptor.Descriptor(
  name='EntryCount',
  full_name='immudb.schema.EntryCount',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='count', full_name='immudb.schema.EntryCount.count', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2270,
  serialized_end=2297,
)


_SIGNATURE = _descriptor.Descriptor(
  name='Signature',
  full_name='immudb.schema.Signature',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='publicKey', full_name='immudb.schema.Signature.publicKey', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='signature', full_name='immudb.schema.Signature.signature', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2299,
  serialized_end=2348,
)


_TXHEADER = _descriptor.Descriptor(
  name='TxHeader',
  full_name='immudb.schema.TxHeader',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='immudb.schema.TxHeader.id', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='prevAlh', full_name='immudb.schema.TxHeader.prevAlh', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ts', full_name='immudb.schema.TxHeader.ts', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nentries', full_name='immudb.schema.TxHeader.nentries', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='eH', full_name='immudb.schema.TxHeader.eH', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='blTxId', full_name='immudb.schema.TxHeader.blTxId', index=5,
      number=6, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='blRoot', full_name='immudb.schema.TxHeader.blRoot', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='immudb.schema.TxHeader.version', index=7,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='immudb.schema.TxHeader.metadata', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2351,
  serialized_end=2526,
)


_TXMETADATA = _descriptor.Descriptor(
  name='TxMetadata',
  full_name='immudb.schema.TxMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2528,
  serialized_end=2540,
)


_LINEARPROOF = _descriptor.Descriptor(
  name='LinearProof',
  full_name='immudb.schema.LinearProof',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sourceTxId', full_name='immudb.schema.LinearProof.sourceTxId', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='TargetTxId', full_name='immudb.schema.LinearProof.TargetTxId', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='terms', full_name='immudb.schema.LinearProof.terms', index=2,
      number=3, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2542,
  serialized_end=2610,
)


_DUALPROOF = _descriptor.Descriptor(
  name='DualProof',
  full_name='immudb.schema.DualProof',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sourceTxHeader', full_name='immudb.schema.DualProof.sourceTxHeader', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='targetTxHeader', full_name='immudb.schema.DualProof.targetTxHeader', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='inclusionProof', full_name='immudb.schema.DualProof.inclusionProof', index=2,
      number=3, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='consistencyProof', full_name='immudb.schema.DualProof.consistencyProof', index=3,
      number=4, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='targetBlTxAlh', full_name='immudb.schema.DualProof.targetBlTxAlh', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lastInclusionProof', full_name='immudb.schema.DualProof.lastInclusionProof', index=5,
      number=6, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='linearProof', full_name='immudb.schema.DualProof.linearProof', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2613,
  serialized_end=2872,
)


_TX = _descriptor.Descriptor(
  name='Tx',
  full_name='immudb.schema.Tx',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='immudb.schema.Tx.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='entries', full_name='immudb.schema.Tx.entries', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='kvEntries', full_name='immudb.schema.Tx.kvEntries', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='zEntries', full_name='immudb.schema.Tx.zEntries', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2875,
  serialized_end=3043,
)


_TXENTRY = _descriptor.Descriptor(
  name='TxEntry',
  full_name='immudb.schema.TxEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.TxEntry.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hValue', full_name='immudb.schema.TxEntry.hValue', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vLen', full_name='immudb.schema.TxEntry.vLen', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='immudb.schema.TxEntry.metadata', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='immudb.schema.TxEntry.value', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3045,
  serialized_end=3157,
)


_KVMETADATA = _descriptor.Descriptor(
  name='KVMetadata',
  full_name='immudb.schema.KVMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='deleted', full_name='immudb.schema.KVMetadata.deleted', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='expiration', full_name='immudb.schema.KVMetadata.expiration', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nonIndexable', full_name='immudb.schema.KVMetadata.nonIndexable', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3159,
  serialized_end=3257,
)


_EXPIRATION = _descriptor.Descriptor(
  name='Expiration',
  full_name='immudb.schema.Expiration',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='expiresAt', full_name='immudb.schema.Expiration.expiresAt', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3259,
  serialized_end=3290,
)


_VERIFIABLETX = _descriptor.Descriptor(
  name='VerifiableTx',
  full_name='immudb.schema.VerifiableTx',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tx', full_name='immudb.schema.VerifiableTx.tx', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dualProof', full_name='immudb.schema.VerifiableTx.dualProof', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='signature', full_name='immudb.schema.VerifiableTx.signature', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3293,
  serialized_end=3428,
)


_VERIFIABLEENTRY = _descriptor.Descriptor(
  name='VerifiableEntry',
  full_name='immudb.schema.VerifiableEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='entry', full_name='immudb.schema.VerifiableEntry.entry', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='verifiableTx', full_name='immudb.schema.VerifiableEntry.verifiableTx', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='inclusionProof', full_name='immudb.schema.VerifiableEntry.inclusionProof', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3431,
  serialized_end=3591,
)


_INCLUSIONPROOF = _descriptor.Descriptor(
  name='InclusionProof',
  full_name='immudb.schema.InclusionProof',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='leaf', full_name='immudb.schema.InclusionProof.leaf', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='width', full_name='immudb.schema.InclusionProof.width', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='terms', full_name='immudb.schema.InclusionProof.terms', index=2,
      number=3, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3593,
  serialized_end=3653,
)


_SETREQUEST = _descriptor.Descriptor(
  name='SetRequest',
  full_name='immudb.schema.SetRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='KVs', full_name='immudb.schema.SetRequest.KVs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='noWait', full_name='immudb.schema.SetRequest.noWait', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='preconditions', full_name='immudb.schema.SetRequest.preconditions', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3655,
  serialized_end=3773,
)


_KEYREQUEST = _descriptor.Descriptor(
  name='KeyRequest',
  full_name='immudb.schema.KeyRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.KeyRequest.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='atTx', full_name='immudb.schema.KeyRequest.atTx', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sinceTx', full_name='immudb.schema.KeyRequest.sinceTx', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='noWait', full_name='immudb.schema.KeyRequest.noWait', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='atRevision', full_name='immudb.schema.KeyRequest.atRevision', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3775,
  serialized_end=3867,
)


_KEYLISTREQUEST = _descriptor.Descriptor(
  name='KeyListRequest',
  full_name='immudb.schema.KeyListRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='keys', full_name='immudb.schema.KeyListRequest.keys', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sinceTx', full_name='immudb.schema.KeyListRequest.sinceTx', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3869,
  serialized_end=3916,
)


_DELETEKEYSREQUEST = _descriptor.Descriptor(
  name='DeleteKeysRequest',
  full_name='immudb.schema.DeleteKeysRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='keys', full_name='immudb.schema.DeleteKeysRequest.keys', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sinceTx', full_name='immudb.schema.DeleteKeysRequest.sinceTx', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='noWait', full_name='immudb.schema.DeleteKeysRequest.noWait', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3918,
  serialized_end=3984,
)


_VERIFIABLESETREQUEST = _descriptor.Descriptor(
  name='VerifiableSetRequest',
  full_name='immudb.schema.VerifiableSetRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='setRequest', full_name='immudb.schema.VerifiableSetRequest.setRequest', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='proveSinceTx', full_name='immudb.schema.VerifiableSetRequest.proveSinceTx', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3986,
  serialized_end=4077,
)


_VERIFIABLEGETREQUEST = _descriptor.Descriptor(
  name='VerifiableGetRequest',
  full_name='immudb.schema.VerifiableGetRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='keyRequest', full_name='immudb.schema.VerifiableGetRequest.keyRequest', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='proveSinceTx', full_name='immudb.schema.VerifiableGetRequest.proveSinceTx', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4079,
  serialized_end=4170,
)


_HEALTHRESPONSE = _descriptor.Descriptor(
  name='HealthResponse',
  full_name='immudb.schema.HealthResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='immudb.schema.HealthResponse.status', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version', full_name='immudb.schema.HealthResponse.version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4172,
  serialized_end=4221,
)


_DATABASEHEALTHRESPONSE = _descriptor.Descriptor(
  name='DatabaseHealthResponse',
  full_name='immudb.schema.DatabaseHealthResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pendingRequests', full_name='immudb.schema.DatabaseHealthResponse.pendingRequests', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lastRequestCompletedAt', full_name='immudb.schema.DatabaseHealthResponse.lastRequestCompletedAt', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4223,
  serialized_end=4304,
)


_IMMUTABLESTATE = _descriptor.Descriptor(
  name='ImmutableState',
  full_name='immudb.schema.ImmutableState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='db', full_name='immudb.schema.ImmutableState.db', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='txId', full_name='immudb.schema.ImmutableState.txId', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='txHash', full_name='immudb.schema.ImmutableState.txHash', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='signature', full_name='immudb.schema.ImmutableState.signature', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4306,
  serialized_end=4409,
)


_REFERENCEREQUEST = _descriptor.Descriptor(
  name='ReferenceRequest',
  full_name='immudb.schema.ReferenceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.ReferenceRequest.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='referencedKey', full_name='immudb.schema.ReferenceRequest.referencedKey', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='atTx', full_name='immudb.schema.ReferenceRequest.atTx', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='boundRef', full_name='immudb.schema.ReferenceRequest.boundRef', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='noWait', full_name='immudb.schema.ReferenceRequest.noWait', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='preconditions', full_name='immudb.schema.ReferenceRequest.preconditions', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4412,
  serialized_end=4566,
)


_VERIFIABLEREFERENCEREQUEST = _descriptor.Descriptor(
  name='VerifiableReferenceRequest',
  full_name='immudb.schema.VerifiableReferenceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='referenceRequest', full_name='immudb.schema.VerifiableReferenceRequest.referenceRequest', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='proveSinceTx', full_name='immudb.schema.VerifiableReferenceRequest.proveSinceTx', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4568,
  serialized_end=4677,
)


_ZADDREQUEST = _descriptor.Descriptor(
  name='ZAddRequest',
  full_name='immudb.schema.ZAddRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='set', full_name='immudb.schema.ZAddRequest.set', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='score', full_name='immudb.schema.ZAddRequest.score', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.ZAddRequest.key', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='atTx', full_name='immudb.schema.ZAddRequest.atTx', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='boundRef', full_name='immudb.schema.ZAddRequest.boundRef', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='noWait', full_name='immudb.schema.ZAddRequest.noWait', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4679,
  serialized_end=4781,
)


_SCORE = _descriptor.Descriptor(
  name='Score',
  full_name='immudb.schema.Score',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='score', full_name='immudb.schema.Score.score', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4783,
  serialized_end=4805,
)


_ZSCANREQUEST = _descriptor.Descriptor(
  name='ZScanRequest',
  full_name='immudb.schema.ZScanRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='set', full_name='immudb.schema.ZScanRequest.set', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seekKey', full_name='immudb.schema.ZScanRequest.seekKey', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seekScore', full_name='immudb.schema.ZScanRequest.seekScore', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seekAtTx', full_name='immudb.schema.ZScanRequest.seekAtTx', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='inclusiveSeek', full_name='immudb.schema.ZScanRequest.inclusiveSeek', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='limit', full_name='immudb.schema.ZScanRequest.limit', index=5,
      number=6, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='desc', full_name='immudb.schema.ZScanRequest.desc', index=6,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='minScore', full_name='immudb.schema.ZScanRequest.minScore', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='maxScore', full_name='immudb.schema.ZScanRequest.maxScore', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sinceTx', full_name='immudb.schema.ZScanRequest.sinceTx', index=9,
      number=10, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='noWait', full_name='immudb.schema.ZScanRequest.noWait', index=10,
      number=11, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4808,
  serialized_end=5054,
)


_HISTORYREQUEST = _descriptor.Descriptor(
  name='HistoryRequest',
  full_name='immudb.schema.HistoryRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.HistoryRequest.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='offset', full_name='immudb.schema.HistoryRequest.offset', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='limit', full_name='immudb.schema.HistoryRequest.limit', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='desc', full_name='immudb.schema.HistoryRequest.desc', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sinceTx', full_name='immudb.schema.HistoryRequest.sinceTx', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5056,
  serialized_end=5147,
)


_VERIFIABLEZADDREQUEST = _descriptor.Descriptor(
  name='VerifiableZAddRequest',
  full_name='immudb.schema.VerifiableZAddRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='zAddRequest', full_name='immudb.schema.VerifiableZAddRequest.zAddRequest', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='proveSinceTx', full_name='immudb.schema.VerifiableZAddRequest.proveSinceTx', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5149,
  serialized_end=5243,
)


_TXREQUEST = _descriptor.Descriptor(
  name='TxRequest',
  full_name='immudb.schema.TxRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tx', full_name='immudb.schema.TxRequest.tx', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='entriesSpec', full_name='immudb.schema.TxRequest.entriesSpec', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sinceTx', full_name='immudb.schema.TxRequest.sinceTx', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='noWait', full_name='immudb.schema.TxRequest.noWait', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5245,
  serialized_end=5350,
)


_ENTRIESSPEC = _descriptor.Descriptor(
  name='EntriesSpec',
  full_name='immudb.schema.EntriesSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='kvEntriesSpec', full_name='immudb.schema.EntriesSpec.kvEntriesSpec', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='zEntriesSpec', full_name='immudb.schema.EntriesSpec.zEntriesSpec', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sqlEntriesSpec', full_name='immudb.schema.EntriesSpec.sqlEntriesSpec', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5353,
  serialized_end=5525,
)


_ENTRYTYPESPEC = _descriptor.Descriptor(
  name='EntryTypeSpec',
  full_name='immudb.schema.EntryTypeSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='action', full_name='immudb.schema.EntryTypeSpec.action', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5527,
  serialized_end=5590,
)


_VERIFIABLETXREQUEST = _descriptor.Descriptor(
  name='VerifiableTxRequest',
  full_name='immudb.schema.VerifiableTxRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tx', full_name='immudb.schema.VerifiableTxRequest.tx', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='proveSinceTx', full_name='immudb.schema.VerifiableTxRequest.proveSinceTx', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='entriesSpec', full_name='immudb.schema.VerifiableTxRequest.entriesSpec', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sinceTx', full_name='immudb.schema.VerifiableTxRequest.sinceTx', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='noWait', full_name='immudb.schema.VerifiableTxRequest.noWait', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5593,
  serialized_end=5730,
)


_TXSCANREQUEST = _descriptor.Descriptor(
  name='TxScanRequest',
  full_name='immudb.schema.TxScanRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='initialTx', full_name='immudb.schema.TxScanRequest.initialTx', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='limit', full_name='immudb.schema.TxScanRequest.limit', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='desc', full_name='immudb.schema.TxScanRequest.desc', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='entriesSpec', full_name='immudb.schema.TxScanRequest.entriesSpec', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sinceTx', full_name='immudb.schema.TxScanRequest.sinceTx', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='noWait', full_name='immudb.schema.TxScanRequest.noWait', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5733,
  serialized_end=5878,
)


_TXLIST = _descriptor.Descriptor(
  name='TxList',
  full_name='immudb.schema.TxList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='txs', full_name='immudb.schema.TxList.txs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5880,
  serialized_end=5920,
)


_EXPORTTXREQUEST = _descriptor.Descriptor(
  name='ExportTxRequest',
  full_name='immudb.schema.ExportTxRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tx', full_name='immudb.schema.ExportTxRequest.tx', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5922,
  serialized_end=5951,
)


_DATABASE = _descriptor.Descriptor(
  name='Database',
  full_name='immudb.schema.Database',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='databaseName', full_name='immudb.schema.Database.databaseName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5953,
  serialized_end=5985,
)


_DATABASESETTINGS = _descriptor.Descriptor(
  name='DatabaseSettings',
  full_name='immudb.schema.DatabaseSettings',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='databaseName', full_name='immudb.schema.DatabaseSettings.databaseName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='replica', full_name='immudb.schema.DatabaseSettings.replica', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='masterDatabase', full_name='immudb.schema.DatabaseSettings.masterDatabase', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='masterAddress', full_name='immudb.schema.DatabaseSettings.masterAddress', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='masterPort', full_name='immudb.schema.DatabaseSettings.masterPort', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='followerUsername', full_name='immudb.schema.DatabaseSettings.followerUsername', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='followerPassword', full_name='immudb.schema.DatabaseSettings.followerPassword', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fileSize', full_name='immudb.schema.DatabaseSettings.fileSize', index=7,
      number=8, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='maxKeyLen', full_name='immudb.schema.DatabaseSettings.maxKeyLen', index=8,
      number=9, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='maxValueLen', full_name='immudb.schema.DatabaseSettings.maxValueLen', index=9,
      number=10, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='maxTxEntries', full_name='immudb.schema.DatabaseSettings.maxTxEntries', index=10,
      number=11, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excludeCommitTime', full_name='immudb.schema.DatabaseSettings.excludeCommitTime', index=11,
      number=12, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5988,
  serialized_end=6271,
)


_CREATEDATABASEREQUEST = _descriptor.Descriptor(
  name='CreateDatabaseRequest',
  full_name='immudb.schema.CreateDatabaseRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='immudb.schema.CreateDatabaseRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='settings', full_name='immudb.schema.CreateDatabaseRequest.settings', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6273,
  serialized_end=6369,
)


_CREATEDATABASERESPONSE = _descriptor.Descriptor(
  name='CreateDatabaseResponse',
  full_name='immudb.schema.CreateDatabaseResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='immudb.schema.CreateDatabaseResponse.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='settings', full_name='immudb.schema.CreateDatabaseResponse.settings', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6371,
  serialized_end=6468,
)


_UPDATEDATABASEREQUEST = _descriptor.Descriptor(
  name='UpdateDatabaseRequest',
  full_name='immudb.schema.UpdateDatabaseRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='database', full_name='immudb.schema.UpdateDatabaseRequest.database', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='settings', full_name='immudb.schema.UpdateDatabaseRequest.settings', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6470,
  serialized_end=6570,
)


_UPDATEDATABASERESPONSE = _descriptor.Descriptor(
  name='UpdateDatabaseResponse',
  full_name='immudb.schema.UpdateDatabaseResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='database', full_name='immudb.schema.UpdateDatabaseResponse.database', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='settings', full_name='immudb.schema.UpdateDatabaseResponse.settings', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6572,
  serialized_end=6673,
)


_DATABASESETTINGSREQUEST = _descriptor.Descriptor(
  name='DatabaseSettingsRequest',
  full_name='immudb.schema.DatabaseSettingsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6675,
  serialized_end=6700,
)


_DATABASESETTINGSRESPONSE = _descriptor.Descriptor(
  name='DatabaseSettingsResponse',
  full_name='immudb.schema.DatabaseSettingsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='database', full_name='immudb.schema.DatabaseSettingsResponse.database', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='settings', full_name='immudb.schema.DatabaseSettingsResponse.settings', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6702,
  serialized_end=6805,
)


_NULLABLEUINT32 = _descriptor.Descriptor(
  name='NullableUint32',
  full_name='immudb.schema.NullableUint32',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='immudb.schema.NullableUint32.value', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6807,
  serialized_end=6838,
)


_NULLABLEUINT64 = _descriptor.Descriptor(
  name='NullableUint64',
  full_name='immudb.schema.NullableUint64',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='immudb.schema.NullableUint64.value', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6840,
  serialized_end=6871,
)


_NULLABLEFLOAT = _descriptor.Descriptor(
  name='NullableFloat',
  full_name='immudb.schema.NullableFloat',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='immudb.schema.NullableFloat.value', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6873,
  serialized_end=6903,
)


_NULLABLEBOOL = _descriptor.Descriptor(
  name='NullableBool',
  full_name='immudb.schema.NullableBool',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='immudb.schema.NullableBool.value', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6905,
  serialized_end=6934,
)


_NULLABLESTRING = _descriptor.Descriptor(
  name='NullableString',
  full_name='immudb.schema.NullableString',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='immudb.schema.NullableString.value', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6936,
  serialized_end=6967,
)


_DATABASENULLABLESETTINGS = _descriptor.Descriptor(
  name='DatabaseNullableSettings',
  full_name='immudb.schema.DatabaseNullableSettings',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='replicationSettings', full_name='immudb.schema.DatabaseNullableSettings.replicationSettings', index=0,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fileSize', full_name='immudb.schema.DatabaseNullableSettings.fileSize', index=1,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='maxKeyLen', full_name='immudb.schema.DatabaseNullableSettings.maxKeyLen', index=2,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='maxValueLen', full_name='immudb.schema.DatabaseNullableSettings.maxValueLen', index=3,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='maxTxEntries', full_name='immudb.schema.DatabaseNullableSettings.maxTxEntries', index=4,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='excludeCommitTime', full_name='immudb.schema.DatabaseNullableSettings.excludeCommitTime', index=5,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='maxConcurrency', full_name='immudb.schema.DatabaseNullableSettings.maxConcurrency', index=6,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='maxIOConcurrency', full_name='immudb.schema.DatabaseNullableSettings.maxIOConcurrency', index=7,
      number=14, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='txLogCacheSize', full_name='immudb.schema.DatabaseNullableSettings.txLogCacheSize', index=8,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vLogMaxOpenedFiles', full_name='immudb.schema.DatabaseNullableSettings.vLogMaxOpenedFiles', index=9,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='txLogMaxOpenedFiles', full_name='immudb.schema.DatabaseNullableSettings.txLogMaxOpenedFiles', index=10,
      number=17, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='commitLogMaxOpenedFiles', full_name='immudb.schema.DatabaseNullableSettings.commitLogMaxOpenedFiles', index=11,
      number=18, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='indexSettings', full_name='immudb.schema.DatabaseNullableSettings.indexSettings', index=12,
      number=19, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='writeTxHeaderVersion', full_name='immudb.schema.DatabaseNullableSettings.writeTxHeaderVersion', index=13,
      number=20, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='autoload', full_name='immudb.schema.DatabaseNullableSettings.autoload', index=14,
      number=21, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6970,
  serialized_end=7848,
)


_REPLICATIONNULLABLESETTINGS = _descriptor.Descriptor(
  name='ReplicationNullableSettings',
  full_name='immudb.schema.ReplicationNullableSettings',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='replica', full_name='immudb.schema.ReplicationNullableSettings.replica', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='masterDatabase', full_name='immudb.schema.ReplicationNullableSettings.masterDatabase', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='masterAddress', full_name='immudb.schema.ReplicationNullableSettings.masterAddress', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='masterPort', full_name='immudb.schema.ReplicationNullableSettings.masterPort', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='followerUsername', full_name='immudb.schema.ReplicationNullableSettings.followerUsername', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='followerPassword', full_name='immudb.schema.ReplicationNullableSettings.followerPassword', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7851,
  serialized_end=8200,
)


_INDEXNULLABLESETTINGS = _descriptor.Descriptor(
  name='IndexNullableSettings',
  full_name='immudb.schema.IndexNullableSettings',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='flushThreshold', full_name='immudb.schema.IndexNullableSettings.flushThreshold', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='syncThreshold', full_name='immudb.schema.IndexNullableSettings.syncThreshold', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cacheSize', full_name='immudb.schema.IndexNullableSettings.cacheSize', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='maxNodeSize', full_name='immudb.schema.IndexNullableSettings.maxNodeSize', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='maxActiveSnapshots', full_name='immudb.schema.IndexNullableSettings.maxActiveSnapshots', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='renewSnapRootAfter', full_name='immudb.schema.IndexNullableSettings.renewSnapRootAfter', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='compactionThld', full_name='immudb.schema.IndexNullableSettings.compactionThld', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='delayDuringCompaction', full_name='immudb.schema.IndexNullableSettings.delayDuringCompaction', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nodesLogMaxOpenedFiles', full_name='immudb.schema.IndexNullableSettings.nodesLogMaxOpenedFiles', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='historyLogMaxOpenedFiles', full_name='immudb.schema.IndexNullableSettings.historyLogMaxOpenedFiles', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='commitLogMaxOpenedFiles', full_name='immudb.schema.IndexNullableSettings.commitLogMaxOpenedFiles', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='flushBufferSize', full_name='immudb.schema.IndexNullableSettings.flushBufferSize', index=11,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cleanupPercentage', full_name='immudb.schema.IndexNullableSettings.cleanupPercentage', index=12,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8203,
  serialized_end=8977,
)


_LOADDATABASEREQUEST = _descriptor.Descriptor(
  name='LoadDatabaseRequest',
  full_name='immudb.schema.LoadDatabaseRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='database', full_name='immudb.schema.LoadDatabaseRequest.database', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8979,
  serialized_end=9018,
)


_LOADDATABASERESPONSE = _descriptor.Descriptor(
  name='LoadDatabaseResponse',
  full_name='immudb.schema.LoadDatabaseResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='database', full_name='immudb.schema.LoadDatabaseResponse.database', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9020,
  serialized_end=9060,
)


_UNLOADDATABASEREQUEST = _descriptor.Descriptor(
  name='UnloadDatabaseRequest',
  full_name='immudb.schema.UnloadDatabaseRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='database', full_name='immudb.schema.UnloadDatabaseRequest.database', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9062,
  serialized_end=9103,
)


_UNLOADDATABASERESPONSE = _descriptor.Descriptor(
  name='UnloadDatabaseResponse',
  full_name='immudb.schema.UnloadDatabaseResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='database', full_name='immudb.schema.UnloadDatabaseResponse.database', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9105,
  serialized_end=9147,
)


_DELETEDATABASEREQUEST = _descriptor.Descriptor(
  name='DeleteDatabaseRequest',
  full_name='immudb.schema.DeleteDatabaseRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='database', full_name='immudb.schema.DeleteDatabaseRequest.database', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9149,
  serialized_end=9190,
)


_DELETEDATABASERESPONSE = _descriptor.Descriptor(
  name='DeleteDatabaseResponse',
  full_name='immudb.schema.DeleteDatabaseResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='database', full_name='immudb.schema.DeleteDatabaseResponse.database', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9192,
  serialized_end=9234,
)


_FLUSHINDEXREQUEST = _descriptor.Descriptor(
  name='FlushIndexRequest',
  full_name='immudb.schema.FlushIndexRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='cleanupPercentage', full_name='immudb.schema.FlushIndexRequest.cleanupPercentage', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='synced', full_name='immudb.schema.FlushIndexRequest.synced', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9236,
  serialized_end=9298,
)


_FLUSHINDEXRESPONSE = _descriptor.Descriptor(
  name='FlushIndexResponse',
  full_name='immudb.schema.FlushIndexResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='database', full_name='immudb.schema.FlushIndexResponse.database', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9300,
  serialized_end=9338,
)


_TABLE = _descriptor.Descriptor(
  name='Table',
  full_name='immudb.schema.Table',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tableName', full_name='immudb.schema.Table.tableName', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9340,
  serialized_end=9366,
)


_SQLGETREQUEST = _descriptor.Descriptor(
  name='SQLGetRequest',
  full_name='immudb.schema.SQLGetRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='table', full_name='immudb.schema.SQLGetRequest.table', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pkValues', full_name='immudb.schema.SQLGetRequest.pkValues', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='atTx', full_name='immudb.schema.SQLGetRequest.atTx', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sinceTx', full_name='immudb.schema.SQLGetRequest.sinceTx', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9368,
  serialized_end=9472,
)


_VERIFIABLESQLGETREQUEST = _descriptor.Descriptor(
  name='VerifiableSQLGetRequest',
  full_name='immudb.schema.VerifiableSQLGetRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sqlGetRequest', full_name='immudb.schema.VerifiableSQLGetRequest.sqlGetRequest', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='proveSinceTx', full_name='immudb.schema.VerifiableSQLGetRequest.proveSinceTx', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9474,
  serialized_end=9574,
)


_SQLENTRY = _descriptor.Descriptor(
  name='SQLEntry',
  full_name='immudb.schema.SQLEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tx', full_name='immudb.schema.SQLEntry.tx', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.SQLEntry.key', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='immudb.schema.SQLEntry.value', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='immudb.schema.SQLEntry.metadata', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9576,
  serialized_end=9671,
)


_VERIFIABLESQLENTRY_COLNAMESBYIDENTRY = _descriptor.Descriptor(
  name='ColNamesByIdEntry',
  full_name='immudb.schema.VerifiableSQLEntry.ColNamesByIdEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.VerifiableSQLEntry.ColNamesByIdEntry.key', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='immudb.schema.VerifiableSQLEntry.ColNamesByIdEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10193,
  serialized_end=10244,
)

_VERIFIABLESQLENTRY_COLIDSBYNAMEENTRY = _descriptor.Descriptor(
  name='ColIdsByNameEntry',
  full_name='immudb.schema.VerifiableSQLEntry.ColIdsByNameEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.VerifiableSQLEntry.ColIdsByNameEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='immudb.schema.VerifiableSQLEntry.ColIdsByNameEntry.value', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10246,
  serialized_end=10297,
)

_VERIFIABLESQLENTRY_COLTYPESBYIDENTRY = _descriptor.Descriptor(
  name='ColTypesByIdEntry',
  full_name='immudb.schema.VerifiableSQLEntry.ColTypesByIdEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.VerifiableSQLEntry.ColTypesByIdEntry.key', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='immudb.schema.VerifiableSQLEntry.ColTypesByIdEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10299,
  serialized_end=10350,
)

_VERIFIABLESQLENTRY_COLLENBYIDENTRY = _descriptor.Descriptor(
  name='ColLenByIdEntry',
  full_name='immudb.schema.VerifiableSQLEntry.ColLenByIdEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.VerifiableSQLEntry.ColLenByIdEntry.key', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='immudb.schema.VerifiableSQLEntry.ColLenByIdEntry.value', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10352,
  serialized_end=10401,
)

_VERIFIABLESQLENTRY = _descriptor.Descriptor(
  name='VerifiableSQLEntry',
  full_name='immudb.schema.VerifiableSQLEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sqlEntry', full_name='immudb.schema.VerifiableSQLEntry.sqlEntry', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='verifiableTx', full_name='immudb.schema.VerifiableSQLEntry.verifiableTx', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='inclusionProof', full_name='immudb.schema.VerifiableSQLEntry.inclusionProof', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='DatabaseId', full_name='immudb.schema.VerifiableSQLEntry.DatabaseId', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='TableId', full_name='immudb.schema.VerifiableSQLEntry.TableId', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='PKIDs', full_name='immudb.schema.VerifiableSQLEntry.PKIDs', index=5,
      number=16, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ColNamesById', full_name='immudb.schema.VerifiableSQLEntry.ColNamesById', index=6,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ColIdsByName', full_name='immudb.schema.VerifiableSQLEntry.ColIdsByName', index=7,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ColTypesById', full_name='immudb.schema.VerifiableSQLEntry.ColTypesById', index=8,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ColLenById', full_name='immudb.schema.VerifiableSQLEntry.ColLenById', index=9,
      number=11, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_VERIFIABLESQLENTRY_COLNAMESBYIDENTRY, _VERIFIABLESQLENTRY_COLIDSBYNAMEENTRY, _VERIFIABLESQLENTRY_COLTYPESBYIDENTRY, _VERIFIABLESQLENTRY_COLLENBYIDENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9674,
  serialized_end=10407,
)


_USEDATABASEREPLY = _descriptor.Descriptor(
  name='UseDatabaseReply',
  full_name='immudb.schema.UseDatabaseReply',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='token', full_name='immudb.schema.UseDatabaseReply.token', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10409,
  serialized_end=10442,
)


_CHANGEPERMISSIONREQUEST = _descriptor.Descriptor(
  name='ChangePermissionRequest',
  full_name='immudb.schema.ChangePermissionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='action', full_name='immudb.schema.ChangePermissionRequest.action', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='username', full_name='immudb.schema.ChangePermissionRequest.username', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='database', full_name='immudb.schema.ChangePermissionRequest.database', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='permission', full_name='immudb.schema.ChangePermissionRequest.permission', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10445,
  serialized_end=10575,
)


_SETACTIVEUSERREQUEST = _descriptor.Descriptor(
  name='SetActiveUserRequest',
  full_name='immudb.schema.SetActiveUserRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='active', full_name='immudb.schema.SetActiveUserRequest.active', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='username', full_name='immudb.schema.SetActiveUserRequest.username', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10577,
  serialized_end=10633,
)


_DATABASELISTRESPONSE = _descriptor.Descriptor(
  name='DatabaseListResponse',
  full_name='immudb.schema.DatabaseListResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='databases', full_name='immudb.schema.DatabaseListResponse.databases', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10635,
  serialized_end=10701,
)


_DATABASELISTREQUESTV2 = _descriptor.Descriptor(
  name='DatabaseListRequestV2',
  full_name='immudb.schema.DatabaseListRequestV2',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10703,
  serialized_end=10726,
)


_DATABASELISTRESPONSEV2 = _descriptor.Descriptor(
  name='DatabaseListResponseV2',
  full_name='immudb.schema.DatabaseListResponseV2',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='databases', full_name='immudb.schema.DatabaseListResponseV2.databases', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10728,
  serialized_end=10808,
)


_DATABASEWITHSETTINGS = _descriptor.Descriptor(
  name='DatabaseWithSettings',
  full_name='immudb.schema.DatabaseWithSettings',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='immudb.schema.DatabaseWithSettings.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='settings', full_name='immudb.schema.DatabaseWithSettings.settings', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='loaded', full_name='immudb.schema.DatabaseWithSettings.loaded', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10810,
  serialized_end=10921,
)


_CHUNK = _descriptor.Descriptor(
  name='Chunk',
  full_name='immudb.schema.Chunk',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='content', full_name='immudb.schema.Chunk.content', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10923,
  serialized_end=10947,
)


_USESNAPSHOTREQUEST = _descriptor.Descriptor(
  name='UseSnapshotRequest',
  full_name='immudb.schema.UseSnapshotRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sinceTx', full_name='immudb.schema.UseSnapshotRequest.sinceTx', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='asBeforeTx', full_name='immudb.schema.UseSnapshotRequest.asBeforeTx', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10949,
  serialized_end=11006,
)


_SQLEXECREQUEST = _descriptor.Descriptor(
  name='SQLExecRequest',
  full_name='immudb.schema.SQLExecRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sql', full_name='immudb.schema.SQLExecRequest.sql', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='params', full_name='immudb.schema.SQLExecRequest.params', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='noWait', full_name='immudb.schema.SQLExecRequest.noWait', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11008,
  serialized_end=11096,
)


_SQLQUERYREQUEST = _descriptor.Descriptor(
  name='SQLQueryRequest',
  full_name='immudb.schema.SQLQueryRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sql', full_name='immudb.schema.SQLQueryRequest.sql', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='params', full_name='immudb.schema.SQLQueryRequest.params', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='reuseSnapshot', full_name='immudb.schema.SQLQueryRequest.reuseSnapshot', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11098,
  serialized_end=11194,
)


_NAMEDPARAM = _descriptor.Descriptor(
  name='NamedParam',
  full_name='immudb.schema.NamedParam',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='immudb.schema.NamedParam.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='immudb.schema.NamedParam.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11196,
  serialized_end=11262,
)


_SQLEXECRESULT = _descriptor.Descriptor(
  name='SQLExecResult',
  full_name='immudb.schema.SQLExecResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='txs', full_name='immudb.schema.SQLExecResult.txs', index=0,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ongoingTx', full_name='immudb.schema.SQLExecResult.ongoingTx', index=1,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11264,
  serialized_end=11342,
)


_COMMITTEDSQLTX_LASTINSERTEDPKSENTRY = _descriptor.Descriptor(
  name='LastInsertedPKsEntry',
  full_name='immudb.schema.CommittedSQLTx.LastInsertedPKsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.CommittedSQLTx.LastInsertedPKsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='immudb.schema.CommittedSQLTx.LastInsertedPKsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11581,
  serialized_end=11660,
)

_COMMITTEDSQLTX_FIRSTINSERTEDPKSENTRY = _descriptor.Descriptor(
  name='FirstInsertedPKsEntry',
  full_name='immudb.schema.CommittedSQLTx.FirstInsertedPKsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='immudb.schema.CommittedSQLTx.FirstInsertedPKsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='immudb.schema.CommittedSQLTx.FirstInsertedPKsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11662,
  serialized_end=11742,
)

_COMMITTEDSQLTX = _descriptor.Descriptor(
  name='CommittedSQLTx',
  full_name='immudb.schema.CommittedSQLTx',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='header', full_name='immudb.schema.CommittedSQLTx.header', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='updatedRows', full_name='immudb.schema.CommittedSQLTx.updatedRows', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lastInsertedPKs', full_name='immudb.schema.CommittedSQLTx.lastInsertedPKs', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='firstInsertedPKs', full_name='immudb.schema.CommittedSQLTx.firstInsertedPKs', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_COMMITTEDSQLTX_LASTINSERTEDPKSENTRY, _COMMITTEDSQLTX_FIRSTINSERTEDPKSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11345,
  serialized_end=11742,
)


_SQLQUERYRESULT = _descriptor.Descriptor(
  name='SQLQueryResult',
  full_name='immudb.schema.SQLQueryResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='columns', full_name='immudb.schema.SQLQueryResult.columns', index=0,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rows', full_name='immudb.schema.SQLQueryResult.rows', index=1,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11744,
  serialized_end=11834,
)


_COLUMN = _descriptor.Descriptor(
  name='Column',
  full_name='immudb.schema.Column',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='immudb.schema.Column.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='immudb.schema.Column.type', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11836,
  serialized_end=11872,
)


_ROW = _descriptor.Descriptor(
  name='Row',
  full_name='immudb.schema.Row',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='columns', full_name='immudb.schema.Row.columns', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values', full_name='immudb.schema.Row.values', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11874,
  serialized_end=11937,
)


_SQLVALUE = _descriptor.Descriptor(
  name='SQLValue',
  full_name='immudb.schema.SQLValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='null', full_name='immudb.schema.SQLValue.null', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='n', full_name='immudb.schema.SQLValue.n', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='s', full_name='immudb.schema.SQLValue.s', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='b', full_name='immudb.schema.SQLValue.b', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='bs', full_name='immudb.schema.SQLValue.bs', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ts', full_name='immudb.schema.SQLValue.ts', index=5,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='value', full_name='immudb.schema.SQLValue.value',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=11940,
  serialized_end=12070,
)


_NEWTXREQUEST = _descriptor.Descriptor(
  name='NewTxRequest',
  full_name='immudb.schema.NewTxRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='mode', full_name='immudb.schema.NewTxRequest.mode', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12072,
  serialized_end=12123,
)


_NEWTXRESPONSE = _descriptor.Descriptor(
  name='NewTxResponse',
  full_name='immudb.schema.NewTxResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='transactionID', full_name='immudb.schema.NewTxResponse.transactionID', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12125,
  serialized_end=12163,
)


_ERRORINFO = _descriptor.Descriptor(
  name='ErrorInfo',
  full_name='immudb.schema.ErrorInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='immudb.schema.ErrorInfo.code', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cause', full_name='immudb.schema.ErrorInfo.cause', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12165,
  serialized_end=12205,
)


_DEBUGINFO = _descriptor.Descriptor(
  name='DebugInfo',
  full_name='immudb.schema.DebugInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stack', full_name='immudb.schema.DebugInfo.stack', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12207,
  serialized_end=12233,
)


_RETRYINFO = _descriptor.Descriptor(
  name='RetryInfo',
  full_name='immudb.schema.RetryInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='retry_delay', full_name='immudb.schema.RetryInfo.retry_delay', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12235,
  serialized_end=12267,
)

_USER.fields_by_name['permissions'].message_type = _PERMISSION
_USERLIST.fields_by_name['users'].message_type = _USER
_PRECONDITION_KEYMUSTEXISTPRECONDITION.containing_type = _PRECONDITION
_PRECONDITION_KEYMUSTNOTEXISTPRECONDITION.containing_type = _PRECONDITION
_PRECONDITION_KEYNOTMODIFIEDAFTERTXPRECONDITION.containing_type = _PRECONDITION
_PRECONDITION.fields_by_name['keyMustExist'].message_type = _PRECONDITION_KEYMUSTEXISTPRECONDITION
_PRECONDITION.fields_by_name['keyMustNotExist'].message_type = _PRECONDITION_KEYMUSTNOTEXISTPRECONDITION
_PRECONDITION.fields_by_name['keyNotModifiedAfterTX'].message_type = _PRECONDITION_KEYNOTMODIFIEDAFTERTXPRECONDITION
_PRECONDITION.oneofs_by_name['precondition'].fields.append(
  _PRECONDITION.fields_by_name['keyMustExist'])
_PRECONDITION.fields_by_name['keyMustExist'].containing_oneof = _PRECONDITION.oneofs_by_name['precondition']
_PRECONDITION.oneofs_by_name['precondition'].fields.append(
  _PRECONDITION.fields_by_name['keyMustNotExist'])
_PRECONDITION.fields_by_name['keyMustNotExist'].containing_oneof = _PRECONDITION.oneofs_by_name['precondition']
_PRECONDITION.oneofs_by_name['precondition'].fields.append(
  _PRECONDITION.fields_by_name['keyNotModifiedAfterTX'])
_PRECONDITION.fields_by_name['keyNotModifiedAfterTX'].containing_oneof = _PRECONDITION.oneofs_by_name['precondition']
_KEYVALUE.fields_by_name['metadata'].message_type = _KVMETADATA
_ENTRY.fields_by_name['referencedBy'].message_type = _REFERENCE
_ENTRY.fields_by_name['metadata'].message_type = _KVMETADATA
_REFERENCE.fields_by_name['metadata'].message_type = _KVMETADATA
_OP.fields_by_name['kv'].message_type = _KEYVALUE
_OP.fields_by_name['zAdd'].message_type = _ZADDREQUEST
_OP.fields_by_name['ref'].message_type = _REFERENCEREQUEST
_OP.oneofs_by_name['operation'].fields.append(
  _OP.fields_by_name['kv'])
_OP.fields_by_name['kv'].containing_oneof = _OP.oneofs_by_name['operation']
_OP.oneofs_by_name['operation'].fields.append(
  _OP.fields_by_name['zAdd'])
_OP.fields_by_name['zAdd'].containing_oneof = _OP.oneofs_by_name['operation']
_OP.oneofs_by_name['operation'].fields.append(
  _OP.fields_by_name['ref'])
_OP.fields_by_name['ref'].containing_oneof = _OP.oneofs_by_name['operation']
_EXECALLREQUEST.fields_by_name['Operations'].message_type = _OP
_EXECALLREQUEST.fields_by_name['preconditions'].message_type = _PRECONDITION
_ENTRIES.fields_by_name['entries'].message_type = _ENTRY
_ZENTRY.fields_by_name['entry'].message_type = _ENTRY
_ZENTRIES.fields_by_name['entries'].message_type = _ZENTRY
_TXHEADER.fields_by_name['metadata'].message_type = _TXMETADATA
_DUALPROOF.fields_by_name['sourceTxHeader'].message_type = _TXHEADER
_DUALPROOF.fields_by_name['targetTxHeader'].message_type = _TXHEADER
_DUALPROOF.fields_by_name['linearProof'].message_type = _LINEARPROOF
_TX.fields_by_name['header'].message_type = _TXHEADER
_TX.fields_by_name['entries'].message_type = _TXENTRY
_TX.fields_by_name['kvEntries'].message_type = _ENTRY
_TX.fields_by_name['zEntries'].message_type = _ZENTRY
_TXENTRY.fields_by_name['metadata'].message_type = _KVMETADATA
_KVMETADATA.fields_by_name['expiration'].message_type = _EXPIRATION
_VERIFIABLETX.fields_by_name['tx'].message_type = _TX
_VERIFIABLETX.fields_by_name['dualProof'].message_type = _DUALPROOF
_VERIFIABLETX.fields_by_name['signature'].message_type = _SIGNATURE
_VERIFIABLEENTRY.fields_by_name['entry'].message_type = _ENTRY
_VERIFIABLEENTRY.fields_by_name['verifiableTx'].message_type = _VERIFIABLETX
_VERIFIABLEENTRY.fields_by_name['inclusionProof'].message_type = _INCLUSIONPROOF
_SETREQUEST.fields_by_name['KVs'].message_type = _KEYVALUE
_SETREQUEST.fields_by_name['preconditions'].message_type = _PRECONDITION
_VERIFIABLESETREQUEST.fields_by_name['setRequest'].message_type = _SETREQUEST
_VERIFIABLEGETREQUEST.fields_by_name['keyRequest'].message_type = _KEYREQUEST
_IMMUTABLESTATE.fields_by_name['signature'].message_type = _SIGNATURE
_REFERENCEREQUEST.fields_by_name['preconditions'].message_type = _PRECONDITION
_VERIFIABLEREFERENCEREQUEST.fields_by_name['referenceRequest'].message_type = _REFERENCEREQUEST
_ZSCANREQUEST.fields_by_name['minScore'].message_type = _SCORE
_ZSCANREQUEST.fields_by_name['maxScore'].message_type = _SCORE
_VERIFIABLEZADDREQUEST.fields_by_name['zAddRequest'].message_type = _ZADDREQUEST
_TXREQUEST.fields_by_name['entriesSpec'].message_type = _ENTRIESSPEC
_ENTRIESSPEC.fields_by_name['kvEntriesSpec'].message_type = _ENTRYTYPESPEC
_ENTRIESSPEC.fields_by_name['zEntriesSpec'].message_type = _ENTRYTYPESPEC
_ENTRIESSPEC.fields_by_name['sqlEntriesSpec'].message_type = _ENTRYTYPESPEC
_ENTRYTYPESPEC.fields_by_name['action'].enum_type = _ENTRYTYPEACTION
_VERIFIABLETXREQUEST.fields_by_name['entriesSpec'].message_type = _ENTRIESSPEC
_TXSCANREQUEST.fields_by_name['entriesSpec'].message_type = _ENTRIESSPEC
_TXLIST.fields_by_name['txs'].message_type = _TX
_CREATEDATABASEREQUEST.fields_by_name['settings'].message_type = _DATABASENULLABLESETTINGS
_CREATEDATABASERESPONSE.fields_by_name['settings'].message_type = _DATABASENULLABLESETTINGS
_UPDATEDATABASEREQUEST.fields_by_name['settings'].message_type = _DATABASENULLABLESETTINGS
_UPDATEDATABASERESPONSE.fields_by_name['settings'].message_type = _DATABASENULLABLESETTINGS
_DATABASESETTINGSRESPONSE.fields_by_name['settings'].message_type = _DATABASENULLABLESETTINGS
_DATABASENULLABLESETTINGS.fields_by_name['replicationSettings'].message_type = _REPLICATIONNULLABLESETTINGS
_DATABASENULLABLESETTINGS.fields_by_name['fileSize'].message_type = _NULLABLEUINT32
_DATABASENULLABLESETTINGS.fields_by_name['maxKeyLen'].message_type = _NULLABLEUINT32
_DATABASENULLABLESETTINGS.fields_by_name['maxValueLen'].message_type = _NULLABLEUINT32
_DATABASENULLABLESETTINGS.fields_by_name['maxTxEntries'].message_type = _NULLABLEUINT32
_DATABASENULLABLESETTINGS.fields_by_name['excludeCommitTime'].message_type = _NULLABLEBOOL
_DATABASENULLABLESETTINGS.fields_by_name['maxConcurrency'].message_type = _NULLABLEUINT32
_DATABASENULLABLESETTINGS.fields_by_name['maxIOConcurrency'].message_type = _NULLABLEUINT32
_DATABASENULLABLESETTINGS.fields_by_name['txLogCacheSize'].message_type = _NULLABLEUINT32
_DATABASENULLABLESETTINGS.fields_by_name['vLogMaxOpenedFiles'].message_type = _NULLABLEUINT32
_DATABASENULLABLESETTINGS.fields_by_name['txLogMaxOpenedFiles'].message_type = _NULLABLEUINT32
_DATABASENULLABLESETTINGS.fields_by_name['commitLogMaxOpenedFiles'].message_type = _NULLABLEUINT32
_DATABASENULLABLESETTINGS.fields_by_name['indexSettings'].message_type = _INDEXNULLABLESETTINGS
_DATABASENULLABLESETTINGS.fields_by_name['writeTxHeaderVersion'].message_type = _NULLABLEUINT32
_DATABASENULLABLESETTINGS.fields_by_name['autoload'].message_type = _NULLABLEBOOL
_REPLICATIONNULLABLESETTINGS.fields_by_name['replica'].message_type = _NULLABLEBOOL
_REPLICATIONNULLABLESETTINGS.fields_by_name['masterDatabase'].message_type = _NULLABLESTRING
_REPLICATIONNULLABLESETTINGS.fields_by_name['masterAddress'].message_type = _NULLABLESTRING
_REPLICATIONNULLABLESETTINGS.fields_by_name['masterPort'].message_type = _NULLABLEUINT32
_REPLICATIONNULLABLESETTINGS.fields_by_name['followerUsername'].message_type = _NULLABLESTRING
_REPLICATIONNULLABLESETTINGS.fields_by_name['followerPassword'].message_type = _NULLABLESTRING
_INDEXNULLABLESETTINGS.fields_by_name['flushThreshold'].message_type = _NULLABLEUINT32
_INDEXNULLABLESETTINGS.fields_by_name['syncThreshold'].message_type = _NULLABLEUINT32
_INDEXNULLABLESETTINGS.fields_by_name['cacheSize'].message_type = _NULLABLEUINT32
_INDEXNULLABLESETTINGS.fields_by_name['maxNodeSize'].message_type = _NULLABLEUINT32
_INDEXNULLABLESETTINGS.fields_by_name['maxActiveSnapshots'].message_type = _NULLABLEUINT32
_INDEXNULLABLESETTINGS.fields_by_name['renewSnapRootAfter'].message_type = _NULLABLEUINT64
_INDEXNULLABLESETTINGS.fields_by_name['compactionThld'].message_type = _NULLABLEUINT32
_INDEXNULLABLESETTINGS.fields_by_name['delayDuringCompaction'].message_type = _NULLABLEUINT32
_INDEXNULLABLESETTINGS.fields_by_name['nodesLogMaxOpenedFiles'].message_type = _NULLABLEUINT32
_INDEXNULLABLESETTINGS.fields_by_name['historyLogMaxOpenedFiles'].message_type = _NULLABLEUINT32
_INDEXNULLABLESETTINGS.fields_by_name['commitLogMaxOpenedFiles'].message_type = _NULLABLEUINT32
_INDEXNULLABLESETTINGS.fields_by_name['flushBufferSize'].message_type = _NULLABLEUINT32
_INDEXNULLABLESETTINGS.fields_by_name['cleanupPercentage'].message_type = _NULLABLEFLOAT
_SQLGETREQUEST.fields_by_name['pkValues'].message_type = _SQLVALUE
_VERIFIABLESQLGETREQUEST.fields_by_name['sqlGetRequest'].message_type = _SQLGETREQUEST
_SQLENTRY.fields_by_name['metadata'].message_type = _KVMETADATA
_VERIFIABLESQLENTRY_COLNAMESBYIDENTRY.containing_type = _VERIFIABLESQLENTRY
_VERIFIABLESQLENTRY_COLIDSBYNAMEENTRY.containing_type = _VERIFIABLESQLENTRY
_VERIFIABLESQLENTRY_COLTYPESBYIDENTRY.containing_type = _VERIFIABLESQLENTRY
_VERIFIABLESQLENTRY_COLLENBYIDENTRY.containing_type = _VERIFIABLESQLENTRY
_VERIFIABLESQLENTRY.fields_by_name['sqlEntry'].message_type = _SQLENTRY
_VERIFIABLESQLENTRY.fields_by_name['verifiableTx'].message_type = _VERIFIABLETX
_VERIFIABLESQLENTRY.fields_by_name['inclusionProof'].message_type = _INCLUSIONPROOF
_VERIFIABLESQLENTRY.fields_by_name['ColNamesById'].message_type = _VERIFIABLESQLENTRY_COLNAMESBYIDENTRY
_VERIFIABLESQLENTRY.fields_by_name['ColIdsByName'].message_type = _VERIFIABLESQLENTRY_COLIDSBYNAMEENTRY
_VERIFIABLESQLENTRY.fields_by_name['ColTypesById'].message_type = _VERIFIABLESQLENTRY_COLTYPESBYIDENTRY
_VERIFIABLESQLENTRY.fields_by_name['ColLenById'].message_type = _VERIFIABLESQLENTRY_COLLENBYIDENTRY
_CHANGEPERMISSIONREQUEST.fields_by_name['action'].enum_type = _PERMISSIONACTION
_DATABASELISTRESPONSE.fields_by_name['databases'].message_type = _DATABASE
_DATABASELISTRESPONSEV2.fields_by_name['databases'].message_type = _DATABASEWITHSETTINGS
_DATABASEWITHSETTINGS.fields_by_name['settings'].message_type = _DATABASENULLABLESETTINGS
_SQLEXECREQUEST.fields_by_name['params'].message_type = _NAMEDPARAM
_SQLQUERYREQUEST.fields_by_name['params'].message_type = _NAMEDPARAM
_NAMEDPARAM.fields_by_name['value'].message_type = _SQLVALUE
_SQLEXECRESULT.fields_by_name['txs'].message_type = _COMMITTEDSQLTX
_COMMITTEDSQLTX_LASTINSERTEDPKSENTRY.fields_by_name['value'].message_type = _SQLVALUE
_COMMITTEDSQLTX_LASTINSERTEDPKSENTRY.containing_type = _COMMITTEDSQLTX
_COMMITTEDSQLTX_FIRSTINSERTEDPKSENTRY.fields_by_name['value'].message_type = _SQLVALUE
_COMMITTEDSQLTX_FIRSTINSERTEDPKSENTRY.containing_type = _COMMITTEDSQLTX
_COMMITTEDSQLTX.fields_by_name['header'].message_type = _TXHEADER
_COMMITTEDSQLTX.fields_by_name['lastInsertedPKs'].message_type = _COMMITTEDSQLTX_LASTINSERTEDPKSENTRY
_COMMITTEDSQLTX.fields_by_name['firstInsertedPKs'].message_type = _COMMITTEDSQLTX_FIRSTINSERTEDPKSENTRY
_SQLQUERYRESULT.fields_by_name['columns'].message_type = _COLUMN
_SQLQUERYRESULT.fields_by_name['rows'].message_type = _ROW
_ROW.fields_by_name['values'].message_type = _SQLVALUE
_SQLVALUE.fields_by_name['null'].enum_type = google_dot_protobuf_dot_struct__pb2._NULLVALUE
_SQLVALUE.oneofs_by_name['value'].fields.append(
  _SQLVALUE.fields_by_name['null'])
_SQLVALUE.fields_by_name['null'].containing_oneof = _SQLVALUE.oneofs_by_name['value']
_SQLVALUE.oneofs_by_name['value'].fields.append(
  _SQLVALUE.fields_by_name['n'])
_SQLVALUE.fields_by_name['n'].containing_oneof = _SQLVALUE.oneofs_by_name['value']
_SQLVALUE.oneofs_by_name['value'].fields.append(
  _SQLVALUE.fields_by_name['s'])
_SQLVALUE.fields_by_name['s'].containing_oneof = _SQLVALUE.oneofs_by_name['value']
_SQLVALUE.oneofs_by_name['value'].fields.append(
  _SQLVALUE.fields_by_name['b'])
_SQLVALUE.fields_by_name['b'].containing_oneof = _SQLVALUE.oneofs_by_name['value']
_SQLVALUE.oneofs_by_name['value'].fields.append(
  _SQLVALUE.fields_by_name['bs'])
_SQLVALUE.fields_by_name['bs'].containing_oneof = _SQLVALUE.oneofs_by_name['value']
_SQLVALUE.oneofs_by_name['value'].fields.append(
  _SQLVALUE.fields_by_name['ts'])
_SQLVALUE.fields_by_name['ts'].containing_oneof = _SQLVALUE.oneofs_by_name['value']
_NEWTXREQUEST.fields_by_name['mode'].enum_type = _TXMODE
DESCRIPTOR.message_types_by_name['Key'] = _KEY
DESCRIPTOR.message_types_by_name['Permission'] = _PERMISSION
DESCRIPTOR.message_types_by_name['User'] = _USER
DESCRIPTOR.message_types_by_name['UserList'] = _USERLIST
DESCRIPTOR.message_types_by_name['CreateUserRequest'] = _CREATEUSERREQUEST
DESCRIPTOR.message_types_by_name['UserRequest'] = _USERREQUEST
DESCRIPTOR.message_types_by_name['ChangePasswordRequest'] = _CHANGEPASSWORDREQUEST
DESCRIPTOR.message_types_by_name['LoginRequest'] = _LOGINREQUEST
DESCRIPTOR.message_types_by_name['LoginResponse'] = _LOGINRESPONSE
DESCRIPTOR.message_types_by_name['AuthConfig'] = _AUTHCONFIG
DESCRIPTOR.message_types_by_name['MTLSConfig'] = _MTLSCONFIG
DESCRIPTOR.message_types_by_name['OpenSessionRequest'] = _OPENSESSIONREQUEST
DESCRIPTOR.message_types_by_name['OpenSessionResponse'] = _OPENSESSIONRESPONSE
DESCRIPTOR.message_types_by_name['Precondition'] = _PRECONDITION
DESCRIPTOR.message_types_by_name['KeyValue'] = _KEYVALUE
DESCRIPTOR.message_types_by_name['Entry'] = _ENTRY
DESCRIPTOR.message_types_by_name['Reference'] = _REFERENCE
DESCRIPTOR.message_types_by_name['Op'] = _OP
DESCRIPTOR.message_types_by_name['ExecAllRequest'] = _EXECALLREQUEST
DESCRIPTOR.message_types_by_name['Entries'] = _ENTRIES
DESCRIPTOR.message_types_by_name['ZEntry'] = _ZENTRY
DESCRIPTOR.message_types_by_name['ZEntries'] = _ZENTRIES
DESCRIPTOR.message_types_by_name['ScanRequest'] = _SCANREQUEST
DESCRIPTOR.message_types_by_name['KeyPrefix'] = _KEYPREFIX
DESCRIPTOR.message_types_by_name['EntryCount'] = _ENTRYCOUNT
DESCRIPTOR.message_types_by_name['Signature'] = _SIGNATURE
DESCRIPTOR.message_types_by_name['TxHeader'] = _TXHEADER
DESCRIPTOR.message_types_by_name['TxMetadata'] = _TXMETADATA
DESCRIPTOR.message_types_by_name['LinearProof'] = _LINEARPROOF
DESCRIPTOR.message_types_by_name['DualProof'] = _DUALPROOF
DESCRIPTOR.message_types_by_name['Tx'] = _TX
DESCRIPTOR.message_types_by_name['TxEntry'] = _TXENTRY
DESCRIPTOR.message_types_by_name['KVMetadata'] = _KVMETADATA
DESCRIPTOR.message_types_by_name['Expiration'] = _EXPIRATION
DESCRIPTOR.message_types_by_name['VerifiableTx'] = _VERIFIABLETX
DESCRIPTOR.message_types_by_name['VerifiableEntry'] = _VERIFIABLEENTRY
DESCRIPTOR.message_types_by_name['InclusionProof'] = _INCLUSIONPROOF
DESCRIPTOR.message_types_by_name['SetRequest'] = _SETREQUEST
DESCRIPTOR.message_types_by_name['KeyRequest'] = _KEYREQUEST
DESCRIPTOR.message_types_by_name['KeyListRequest'] = _KEYLISTREQUEST
DESCRIPTOR.message_types_by_name['DeleteKeysRequest'] = _DELETEKEYSREQUEST
DESCRIPTOR.message_types_by_name['VerifiableSetRequest'] = _VERIFIABLESETREQUEST
DESCRIPTOR.message_types_by_name['VerifiableGetRequest'] = _VERIFIABLEGETREQUEST
DESCRIPTOR.message_types_by_name['HealthResponse'] = _HEALTHRESPONSE
DESCRIPTOR.message_types_by_name['DatabaseHealthResponse'] = _DATABASEHEALTHRESPONSE
DESCRIPTOR.message_types_by_name['ImmutableState'] = _IMMUTABLESTATE
DESCRIPTOR.message_types_by_name['ReferenceRequest'] = _REFERENCEREQUEST
DESCRIPTOR.message_types_by_name['VerifiableReferenceRequest'] = _VERIFIABLEREFERENCEREQUEST
DESCRIPTOR.message_types_by_name['ZAddRequest'] = _ZADDREQUEST
DESCRIPTOR.message_types_by_name['Score'] = _SCORE
DESCRIPTOR.message_types_by_name['ZScanRequest'] = _ZSCANREQUEST
DESCRIPTOR.message_types_by_name['HistoryRequest'] = _HISTORYREQUEST
DESCRIPTOR.message_types_by_name['VerifiableZAddRequest'] = _VERIFIABLEZADDREQUEST
DESCRIPTOR.message_types_by_name['TxRequest'] = _TXREQUEST
DESCRIPTOR.message_types_by_name['EntriesSpec'] = _ENTRIESSPEC
DESCRIPTOR.message_types_by_name['EntryTypeSpec'] = _ENTRYTYPESPEC
DESCRIPTOR.message_types_by_name['VerifiableTxRequest'] = _VERIFIABLETXREQUEST
DESCRIPTOR.message_types_by_name['TxScanRequest'] = _TXSCANREQUEST
DESCRIPTOR.message_types_by_name['TxList'] = _TXLIST
DESCRIPTOR.message_types_by_name['ExportTxRequest'] = _EXPORTTXREQUEST
DESCRIPTOR.message_types_by_name['Database'] = _DATABASE
DESCRIPTOR.message_types_by_name['DatabaseSettings'] = _DATABASESETTINGS
DESCRIPTOR.message_types_by_name['CreateDatabaseRequest'] = _CREATEDATABASEREQUEST
DESCRIPTOR.message_types_by_name['CreateDatabaseResponse'] = _CREATEDATABASERESPONSE
DESCRIPTOR.message_types_by_name['UpdateDatabaseRequest'] = _UPDATEDATABASEREQUEST
DESCRIPTOR.message_types_by_name['UpdateDatabaseResponse'] = _UPDATEDATABASERESPONSE
DESCRIPTOR.message_types_by_name['DatabaseSettingsRequest'] = _DATABASESETTINGSREQUEST
DESCRIPTOR.message_types_by_name['DatabaseSettingsResponse'] = _DATABASESETTINGSRESPONSE
DESCRIPTOR.message_types_by_name['NullableUint32'] = _NULLABLEUINT32
DESCRIPTOR.message_types_by_name['NullableUint64'] = _NULLABLEUINT64
DESCRIPTOR.message_types_by_name['NullableFloat'] = _NULLABLEFLOAT
DESCRIPTOR.message_types_by_name['NullableBool'] = _NULLABLEBOOL
DESCRIPTOR.message_types_by_name['NullableString'] = _NULLABLESTRING
DESCRIPTOR.message_types_by_name['DatabaseNullableSettings'] = _DATABASENULLABLESETTINGS
DESCRIPTOR.message_types_by_name['ReplicationNullableSettings'] = _REPLICATIONNULLABLESETTINGS
DESCRIPTOR.message_types_by_name['IndexNullableSettings'] = _INDEXNULLABLESETTINGS
DESCRIPTOR.message_types_by_name['LoadDatabaseRequest'] = _LOADDATABASEREQUEST
DESCRIPTOR.message_types_by_name['LoadDatabaseResponse'] = _LOADDATABASERESPONSE
DESCRIPTOR.message_types_by_name['UnloadDatabaseRequest'] = _UNLOADDATABASEREQUEST
DESCRIPTOR.message_types_by_name['UnloadDatabaseResponse'] = _UNLOADDATABASERESPONSE
DESCRIPTOR.message_types_by_name['DeleteDatabaseRequest'] = _DELETEDATABASEREQUEST
DESCRIPTOR.message_types_by_name['DeleteDatabaseResponse'] = _DELETEDATABASERESPONSE
DESCRIPTOR.message_types_by_name['FlushIndexRequest'] = _FLUSHINDEXREQUEST
DESCRIPTOR.message_types_by_name['FlushIndexResponse'] = _FLUSHINDEXRESPONSE
DESCRIPTOR.message_types_by_name['Table'] = _TABLE
DESCRIPTOR.message_types_by_name['SQLGetRequest'] = _SQLGETREQUEST
DESCRIPTOR.message_types_by_name['VerifiableSQLGetRequest'] = _VERIFIABLESQLGETREQUEST
DESCRIPTOR.message_types_by_name['SQLEntry'] = _SQLENTRY
DESCRIPTOR.message_types_by_name['VerifiableSQLEntry'] = _VERIFIABLESQLENTRY
DESCRIPTOR.message_types_by_name['UseDatabaseReply'] = _USEDATABASEREPLY
DESCRIPTOR.message_types_by_name['ChangePermissionRequest'] = _CHANGEPERMISSIONREQUEST
DESCRIPTOR.message_types_by_name['SetActiveUserRequest'] = _SETACTIVEUSERREQUEST
DESCRIPTOR.message_types_by_name['DatabaseListResponse'] = _DATABASELISTRESPONSE
DESCRIPTOR.message_types_by_name['DatabaseListRequestV2'] = _DATABASELISTREQUESTV2
DESCRIPTOR.message_types_by_name['DatabaseListResponseV2'] = _DATABASELISTRESPONSEV2
DESCRIPTOR.message_types_by_name['DatabaseWithSettings'] = _DATABASEWITHSETTINGS
DESCRIPTOR.message_types_by_name['Chunk'] = _CHUNK
DESCRIPTOR.message_types_by_name['UseSnapshotRequest'] = _USESNAPSHOTREQUEST
DESCRIPTOR.message_types_by_name['SQLExecRequest'] = _SQLEXECREQUEST
DESCRIPTOR.message_types_by_name['SQLQueryRequest'] = _SQLQUERYREQUEST
DESCRIPTOR.message_types_by_name['NamedParam'] = _NAMEDPARAM
DESCRIPTOR.message_types_by_name['SQLExecResult'] = _SQLEXECRESULT
DESCRIPTOR.message_types_by_name['CommittedSQLTx'] = _COMMITTEDSQLTX
DESCRIPTOR.message_types_by_name['SQLQueryResult'] = _SQLQUERYRESULT
DESCRIPTOR.message_types_by_name['Column'] = _COLUMN
DESCRIPTOR.message_types_by_name['Row'] = _ROW
DESCRIPTOR.message_types_by_name['SQLValue'] = _SQLVALUE
DESCRIPTOR.message_types_by_name['NewTxRequest'] = _NEWTXREQUEST
DESCRIPTOR.message_types_by_name['NewTxResponse'] = _NEWTXRESPONSE
DESCRIPTOR.message_types_by_name['ErrorInfo'] = _ERRORINFO
DESCRIPTOR.message_types_by_name['DebugInfo'] = _DEBUGINFO
DESCRIPTOR.message_types_by_name['RetryInfo'] = _RETRYINFO
DESCRIPTOR.enum_types_by_name['EntryTypeAction'] = _ENTRYTYPEACTION
DESCRIPTOR.enum_types_by_name['PermissionAction'] = _PERMISSIONACTION
DESCRIPTOR.enum_types_by_name['TxMode'] = _TXMODE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Key = _reflection.GeneratedProtocolMessageType('Key', (_message.Message,), {
  'DESCRIPTOR' : _KEY,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.Key)
  })
_sym_db.RegisterMessage(Key)

Permission = _reflection.GeneratedProtocolMessageType('Permission', (_message.Message,), {
  'DESCRIPTOR' : _PERMISSION,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.Permission)
  })
_sym_db.RegisterMessage(Permission)

User = _reflection.GeneratedProtocolMessageType('User', (_message.Message,), {
  'DESCRIPTOR' : _USER,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.User)
  })
_sym_db.RegisterMessage(User)

UserList = _reflection.GeneratedProtocolMessageType('UserList', (_message.Message,), {
  'DESCRIPTOR' : _USERLIST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.UserList)
  })
_sym_db.RegisterMessage(UserList)

CreateUserRequest = _reflection.GeneratedProtocolMessageType('CreateUserRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEUSERREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.CreateUserRequest)
  })
_sym_db.RegisterMessage(CreateUserRequest)

UserRequest = _reflection.GeneratedProtocolMessageType('UserRequest', (_message.Message,), {
  'DESCRIPTOR' : _USERREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.UserRequest)
  })
_sym_db.RegisterMessage(UserRequest)

ChangePasswordRequest = _reflection.GeneratedProtocolMessageType('ChangePasswordRequest', (_message.Message,), {
  'DESCRIPTOR' : _CHANGEPASSWORDREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.ChangePasswordRequest)
  })
_sym_db.RegisterMessage(ChangePasswordRequest)

LoginRequest = _reflection.GeneratedProtocolMessageType('LoginRequest', (_message.Message,), {
  'DESCRIPTOR' : _LOGINREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.LoginRequest)
  })
_sym_db.RegisterMessage(LoginRequest)

LoginResponse = _reflection.GeneratedProtocolMessageType('LoginResponse', (_message.Message,), {
  'DESCRIPTOR' : _LOGINRESPONSE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.LoginResponse)
  })
_sym_db.RegisterMessage(LoginResponse)

AuthConfig = _reflection.GeneratedProtocolMessageType('AuthConfig', (_message.Message,), {
  'DESCRIPTOR' : _AUTHCONFIG,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.AuthConfig)
  })
_sym_db.RegisterMessage(AuthConfig)

MTLSConfig = _reflection.GeneratedProtocolMessageType('MTLSConfig', (_message.Message,), {
  'DESCRIPTOR' : _MTLSCONFIG,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.MTLSConfig)
  })
_sym_db.RegisterMessage(MTLSConfig)

OpenSessionRequest = _reflection.GeneratedProtocolMessageType('OpenSessionRequest', (_message.Message,), {
  'DESCRIPTOR' : _OPENSESSIONREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.OpenSessionRequest)
  })
_sym_db.RegisterMessage(OpenSessionRequest)

OpenSessionResponse = _reflection.GeneratedProtocolMessageType('OpenSessionResponse', (_message.Message,), {
  'DESCRIPTOR' : _OPENSESSIONRESPONSE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.OpenSessionResponse)
  })
_sym_db.RegisterMessage(OpenSessionResponse)

Precondition = _reflection.GeneratedProtocolMessageType('Precondition', (_message.Message,), {

  'KeyMustExistPrecondition' : _reflection.GeneratedProtocolMessageType('KeyMustExistPrecondition', (_message.Message,), {
    'DESCRIPTOR' : _PRECONDITION_KEYMUSTEXISTPRECONDITION,
    '__module__' : 'schema_pb2'
    # @@protoc_insertion_point(class_scope:immudb.schema.Precondition.KeyMustExistPrecondition)
    })
  ,

  'KeyMustNotExistPrecondition' : _reflection.GeneratedProtocolMessageType('KeyMustNotExistPrecondition', (_message.Message,), {
    'DESCRIPTOR' : _PRECONDITION_KEYMUSTNOTEXISTPRECONDITION,
    '__module__' : 'schema_pb2'
    # @@protoc_insertion_point(class_scope:immudb.schema.Precondition.KeyMustNotExistPrecondition)
    })
  ,

  'KeyNotModifiedAfterTXPrecondition' : _reflection.GeneratedProtocolMessageType('KeyNotModifiedAfterTXPrecondition', (_message.Message,), {
    'DESCRIPTOR' : _PRECONDITION_KEYNOTMODIFIEDAFTERTXPRECONDITION,
    '__module__' : 'schema_pb2'
    # @@protoc_insertion_point(class_scope:immudb.schema.Precondition.KeyNotModifiedAfterTXPrecondition)
    })
  ,
  'DESCRIPTOR' : _PRECONDITION,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.Precondition)
  })
_sym_db.RegisterMessage(Precondition)
_sym_db.RegisterMessage(Precondition.KeyMustExistPrecondition)
_sym_db.RegisterMessage(Precondition.KeyMustNotExistPrecondition)
_sym_db.RegisterMessage(Precondition.KeyNotModifiedAfterTXPrecondition)

KeyValue = _reflection.GeneratedProtocolMessageType('KeyValue', (_message.Message,), {
  'DESCRIPTOR' : _KEYVALUE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.KeyValue)
  })
_sym_db.RegisterMessage(KeyValue)

Entry = _reflection.GeneratedProtocolMessageType('Entry', (_message.Message,), {
  'DESCRIPTOR' : _ENTRY,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.Entry)
  })
_sym_db.RegisterMessage(Entry)

Reference = _reflection.GeneratedProtocolMessageType('Reference', (_message.Message,), {
  'DESCRIPTOR' : _REFERENCE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.Reference)
  })
_sym_db.RegisterMessage(Reference)

Op = _reflection.GeneratedProtocolMessageType('Op', (_message.Message,), {
  'DESCRIPTOR' : _OP,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.Op)
  })
_sym_db.RegisterMessage(Op)

ExecAllRequest = _reflection.GeneratedProtocolMessageType('ExecAllRequest', (_message.Message,), {
  'DESCRIPTOR' : _EXECALLREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.ExecAllRequest)
  })
_sym_db.RegisterMessage(ExecAllRequest)

Entries = _reflection.GeneratedProtocolMessageType('Entries', (_message.Message,), {
  'DESCRIPTOR' : _ENTRIES,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.Entries)
  })
_sym_db.RegisterMessage(Entries)

ZEntry = _reflection.GeneratedProtocolMessageType('ZEntry', (_message.Message,), {
  'DESCRIPTOR' : _ZENTRY,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.ZEntry)
  })
_sym_db.RegisterMessage(ZEntry)

ZEntries = _reflection.GeneratedProtocolMessageType('ZEntries', (_message.Message,), {
  'DESCRIPTOR' : _ZENTRIES,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.ZEntries)
  })
_sym_db.RegisterMessage(ZEntries)

ScanRequest = _reflection.GeneratedProtocolMessageType('ScanRequest', (_message.Message,), {
  'DESCRIPTOR' : _SCANREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.ScanRequest)
  })
_sym_db.RegisterMessage(ScanRequest)

KeyPrefix = _reflection.GeneratedProtocolMessageType('KeyPrefix', (_message.Message,), {
  'DESCRIPTOR' : _KEYPREFIX,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.KeyPrefix)
  })
_sym_db.RegisterMessage(KeyPrefix)

EntryCount = _reflection.GeneratedProtocolMessageType('EntryCount', (_message.Message,), {
  'DESCRIPTOR' : _ENTRYCOUNT,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.EntryCount)
  })
_sym_db.RegisterMessage(EntryCount)

Signature = _reflection.GeneratedProtocolMessageType('Signature', (_message.Message,), {
  'DESCRIPTOR' : _SIGNATURE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.Signature)
  })
_sym_db.RegisterMessage(Signature)

TxHeader = _reflection.GeneratedProtocolMessageType('TxHeader', (_message.Message,), {
  'DESCRIPTOR' : _TXHEADER,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.TxHeader)
  })
_sym_db.RegisterMessage(TxHeader)

TxMetadata = _reflection.GeneratedProtocolMessageType('TxMetadata', (_message.Message,), {
  'DESCRIPTOR' : _TXMETADATA,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.TxMetadata)
  })
_sym_db.RegisterMessage(TxMetadata)

LinearProof = _reflection.GeneratedProtocolMessageType('LinearProof', (_message.Message,), {
  'DESCRIPTOR' : _LINEARPROOF,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.LinearProof)
  })
_sym_db.RegisterMessage(LinearProof)

DualProof = _reflection.GeneratedProtocolMessageType('DualProof', (_message.Message,), {
  'DESCRIPTOR' : _DUALPROOF,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.DualProof)
  })
_sym_db.RegisterMessage(DualProof)

Tx = _reflection.GeneratedProtocolMessageType('Tx', (_message.Message,), {
  'DESCRIPTOR' : _TX,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.Tx)
  })
_sym_db.RegisterMessage(Tx)

TxEntry = _reflection.GeneratedProtocolMessageType('TxEntry', (_message.Message,), {
  'DESCRIPTOR' : _TXENTRY,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.TxEntry)
  })
_sym_db.RegisterMessage(TxEntry)

KVMetadata = _reflection.GeneratedProtocolMessageType('KVMetadata', (_message.Message,), {
  'DESCRIPTOR' : _KVMETADATA,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.KVMetadata)
  })
_sym_db.RegisterMessage(KVMetadata)

Expiration = _reflection.GeneratedProtocolMessageType('Expiration', (_message.Message,), {
  'DESCRIPTOR' : _EXPIRATION,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.Expiration)
  })
_sym_db.RegisterMessage(Expiration)

VerifiableTx = _reflection.GeneratedProtocolMessageType('VerifiableTx', (_message.Message,), {
  'DESCRIPTOR' : _VERIFIABLETX,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.VerifiableTx)
  })
_sym_db.RegisterMessage(VerifiableTx)

VerifiableEntry = _reflection.GeneratedProtocolMessageType('VerifiableEntry', (_message.Message,), {
  'DESCRIPTOR' : _VERIFIABLEENTRY,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.VerifiableEntry)
  })
_sym_db.RegisterMessage(VerifiableEntry)

InclusionProof = _reflection.GeneratedProtocolMessageType('InclusionProof', (_message.Message,), {
  'DESCRIPTOR' : _INCLUSIONPROOF,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.InclusionProof)
  })
_sym_db.RegisterMessage(InclusionProof)

SetRequest = _reflection.GeneratedProtocolMessageType('SetRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.SetRequest)
  })
_sym_db.RegisterMessage(SetRequest)

KeyRequest = _reflection.GeneratedProtocolMessageType('KeyRequest', (_message.Message,), {
  'DESCRIPTOR' : _KEYREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.KeyRequest)
  })
_sym_db.RegisterMessage(KeyRequest)

KeyListRequest = _reflection.GeneratedProtocolMessageType('KeyListRequest', (_message.Message,), {
  'DESCRIPTOR' : _KEYLISTREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.KeyListRequest)
  })
_sym_db.RegisterMessage(KeyListRequest)

DeleteKeysRequest = _reflection.GeneratedProtocolMessageType('DeleteKeysRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEKEYSREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.DeleteKeysRequest)
  })
_sym_db.RegisterMessage(DeleteKeysRequest)

VerifiableSetRequest = _reflection.GeneratedProtocolMessageType('VerifiableSetRequest', (_message.Message,), {
  'DESCRIPTOR' : _VERIFIABLESETREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.VerifiableSetRequest)
  })
_sym_db.RegisterMessage(VerifiableSetRequest)

VerifiableGetRequest = _reflection.GeneratedProtocolMessageType('VerifiableGetRequest', (_message.Message,), {
  'DESCRIPTOR' : _VERIFIABLEGETREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.VerifiableGetRequest)
  })
_sym_db.RegisterMessage(VerifiableGetRequest)

HealthResponse = _reflection.GeneratedProtocolMessageType('HealthResponse', (_message.Message,), {
  'DESCRIPTOR' : _HEALTHRESPONSE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.HealthResponse)
  })
_sym_db.RegisterMessage(HealthResponse)

DatabaseHealthResponse = _reflection.GeneratedProtocolMessageType('DatabaseHealthResponse', (_message.Message,), {
  'DESCRIPTOR' : _DATABASEHEALTHRESPONSE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.DatabaseHealthResponse)
  })
_sym_db.RegisterMessage(DatabaseHealthResponse)

ImmutableState = _reflection.GeneratedProtocolMessageType('ImmutableState', (_message.Message,), {
  'DESCRIPTOR' : _IMMUTABLESTATE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.ImmutableState)
  })
_sym_db.RegisterMessage(ImmutableState)

ReferenceRequest = _reflection.GeneratedProtocolMessageType('ReferenceRequest', (_message.Message,), {
  'DESCRIPTOR' : _REFERENCEREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.ReferenceRequest)
  })
_sym_db.RegisterMessage(ReferenceRequest)

VerifiableReferenceRequest = _reflection.GeneratedProtocolMessageType('VerifiableReferenceRequest', (_message.Message,), {
  'DESCRIPTOR' : _VERIFIABLEREFERENCEREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.VerifiableReferenceRequest)
  })
_sym_db.RegisterMessage(VerifiableReferenceRequest)

ZAddRequest = _reflection.GeneratedProtocolMessageType('ZAddRequest', (_message.Message,), {
  'DESCRIPTOR' : _ZADDREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.ZAddRequest)
  })
_sym_db.RegisterMessage(ZAddRequest)

Score = _reflection.GeneratedProtocolMessageType('Score', (_message.Message,), {
  'DESCRIPTOR' : _SCORE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.Score)
  })
_sym_db.RegisterMessage(Score)

ZScanRequest = _reflection.GeneratedProtocolMessageType('ZScanRequest', (_message.Message,), {
  'DESCRIPTOR' : _ZSCANREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.ZScanRequest)
  })
_sym_db.RegisterMessage(ZScanRequest)

HistoryRequest = _reflection.GeneratedProtocolMessageType('HistoryRequest', (_message.Message,), {
  'DESCRIPTOR' : _HISTORYREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.HistoryRequest)
  })
_sym_db.RegisterMessage(HistoryRequest)

VerifiableZAddRequest = _reflection.GeneratedProtocolMessageType('VerifiableZAddRequest', (_message.Message,), {
  'DESCRIPTOR' : _VERIFIABLEZADDREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.VerifiableZAddRequest)
  })
_sym_db.RegisterMessage(VerifiableZAddRequest)

TxRequest = _reflection.GeneratedProtocolMessageType('TxRequest', (_message.Message,), {
  'DESCRIPTOR' : _TXREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.TxRequest)
  })
_sym_db.RegisterMessage(TxRequest)

EntriesSpec = _reflection.GeneratedProtocolMessageType('EntriesSpec', (_message.Message,), {
  'DESCRIPTOR' : _ENTRIESSPEC,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.EntriesSpec)
  })
_sym_db.RegisterMessage(EntriesSpec)

EntryTypeSpec = _reflection.GeneratedProtocolMessageType('EntryTypeSpec', (_message.Message,), {
  'DESCRIPTOR' : _ENTRYTYPESPEC,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.EntryTypeSpec)
  })
_sym_db.RegisterMessage(EntryTypeSpec)

VerifiableTxRequest = _reflection.GeneratedProtocolMessageType('VerifiableTxRequest', (_message.Message,), {
  'DESCRIPTOR' : _VERIFIABLETXREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.VerifiableTxRequest)
  })
_sym_db.RegisterMessage(VerifiableTxRequest)

TxScanRequest = _reflection.GeneratedProtocolMessageType('TxScanRequest', (_message.Message,), {
  'DESCRIPTOR' : _TXSCANREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.TxScanRequest)
  })
_sym_db.RegisterMessage(TxScanRequest)

TxList = _reflection.GeneratedProtocolMessageType('TxList', (_message.Message,), {
  'DESCRIPTOR' : _TXLIST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.TxList)
  })
_sym_db.RegisterMessage(TxList)

ExportTxRequest = _reflection.GeneratedProtocolMessageType('ExportTxRequest', (_message.Message,), {
  'DESCRIPTOR' : _EXPORTTXREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.ExportTxRequest)
  })
_sym_db.RegisterMessage(ExportTxRequest)

Database = _reflection.GeneratedProtocolMessageType('Database', (_message.Message,), {
  'DESCRIPTOR' : _DATABASE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.Database)
  })
_sym_db.RegisterMessage(Database)

DatabaseSettings = _reflection.GeneratedProtocolMessageType('DatabaseSettings', (_message.Message,), {
  'DESCRIPTOR' : _DATABASESETTINGS,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.DatabaseSettings)
  })
_sym_db.RegisterMessage(DatabaseSettings)

CreateDatabaseRequest = _reflection.GeneratedProtocolMessageType('CreateDatabaseRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEDATABASEREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.CreateDatabaseRequest)
  })
_sym_db.RegisterMessage(CreateDatabaseRequest)

CreateDatabaseResponse = _reflection.GeneratedProtocolMessageType('CreateDatabaseResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEDATABASERESPONSE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.CreateDatabaseResponse)
  })
_sym_db.RegisterMessage(CreateDatabaseResponse)

UpdateDatabaseRequest = _reflection.GeneratedProtocolMessageType('UpdateDatabaseRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEDATABASEREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.UpdateDatabaseRequest)
  })
_sym_db.RegisterMessage(UpdateDatabaseRequest)

UpdateDatabaseResponse = _reflection.GeneratedProtocolMessageType('UpdateDatabaseResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEDATABASERESPONSE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.UpdateDatabaseResponse)
  })
_sym_db.RegisterMessage(UpdateDatabaseResponse)

DatabaseSettingsRequest = _reflection.GeneratedProtocolMessageType('DatabaseSettingsRequest', (_message.Message,), {
  'DESCRIPTOR' : _DATABASESETTINGSREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.DatabaseSettingsRequest)
  })
_sym_db.RegisterMessage(DatabaseSettingsRequest)

DatabaseSettingsResponse = _reflection.GeneratedProtocolMessageType('DatabaseSettingsResponse', (_message.Message,), {
  'DESCRIPTOR' : _DATABASESETTINGSRESPONSE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.DatabaseSettingsResponse)
  })
_sym_db.RegisterMessage(DatabaseSettingsResponse)

NullableUint32 = _reflection.GeneratedProtocolMessageType('NullableUint32', (_message.Message,), {
  'DESCRIPTOR' : _NULLABLEUINT32,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.NullableUint32)
  })
_sym_db.RegisterMessage(NullableUint32)

NullableUint64 = _reflection.GeneratedProtocolMessageType('NullableUint64', (_message.Message,), {
  'DESCRIPTOR' : _NULLABLEUINT64,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.NullableUint64)
  })
_sym_db.RegisterMessage(NullableUint64)

NullableFloat = _reflection.GeneratedProtocolMessageType('NullableFloat', (_message.Message,), {
  'DESCRIPTOR' : _NULLABLEFLOAT,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.NullableFloat)
  })
_sym_db.RegisterMessage(NullableFloat)

NullableBool = _reflection.GeneratedProtocolMessageType('NullableBool', (_message.Message,), {
  'DESCRIPTOR' : _NULLABLEBOOL,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.NullableBool)
  })
_sym_db.RegisterMessage(NullableBool)

NullableString = _reflection.GeneratedProtocolMessageType('NullableString', (_message.Message,), {
  'DESCRIPTOR' : _NULLABLESTRING,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.NullableString)
  })
_sym_db.RegisterMessage(NullableString)

DatabaseNullableSettings = _reflection.GeneratedProtocolMessageType('DatabaseNullableSettings', (_message.Message,), {
  'DESCRIPTOR' : _DATABASENULLABLESETTINGS,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.DatabaseNullableSettings)
  })
_sym_db.RegisterMessage(DatabaseNullableSettings)

ReplicationNullableSettings = _reflection.GeneratedProtocolMessageType('ReplicationNullableSettings', (_message.Message,), {
  'DESCRIPTOR' : _REPLICATIONNULLABLESETTINGS,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.ReplicationNullableSettings)
  })
_sym_db.RegisterMessage(ReplicationNullableSettings)

IndexNullableSettings = _reflection.GeneratedProtocolMessageType('IndexNullableSettings', (_message.Message,), {
  'DESCRIPTOR' : _INDEXNULLABLESETTINGS,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.IndexNullableSettings)
  })
_sym_db.RegisterMessage(IndexNullableSettings)

LoadDatabaseRequest = _reflection.GeneratedProtocolMessageType('LoadDatabaseRequest', (_message.Message,), {
  'DESCRIPTOR' : _LOADDATABASEREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.LoadDatabaseRequest)
  })
_sym_db.RegisterMessage(LoadDatabaseRequest)

LoadDatabaseResponse = _reflection.GeneratedProtocolMessageType('LoadDatabaseResponse', (_message.Message,), {
  'DESCRIPTOR' : _LOADDATABASERESPONSE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.LoadDatabaseResponse)
  })
_sym_db.RegisterMessage(LoadDatabaseResponse)

UnloadDatabaseRequest = _reflection.GeneratedProtocolMessageType('UnloadDatabaseRequest', (_message.Message,), {
  'DESCRIPTOR' : _UNLOADDATABASEREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.UnloadDatabaseRequest)
  })
_sym_db.RegisterMessage(UnloadDatabaseRequest)

UnloadDatabaseResponse = _reflection.GeneratedProtocolMessageType('UnloadDatabaseResponse', (_message.Message,), {
  'DESCRIPTOR' : _UNLOADDATABASERESPONSE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.UnloadDatabaseResponse)
  })
_sym_db.RegisterMessage(UnloadDatabaseResponse)

DeleteDatabaseRequest = _reflection.GeneratedProtocolMessageType('DeleteDatabaseRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEDATABASEREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.DeleteDatabaseRequest)
  })
_sym_db.RegisterMessage(DeleteDatabaseRequest)

DeleteDatabaseResponse = _reflection.GeneratedProtocolMessageType('DeleteDatabaseResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEDATABASERESPONSE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.DeleteDatabaseResponse)
  })
_sym_db.RegisterMessage(DeleteDatabaseResponse)

FlushIndexRequest = _reflection.GeneratedProtocolMessageType('FlushIndexRequest', (_message.Message,), {
  'DESCRIPTOR' : _FLUSHINDEXREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.FlushIndexRequest)
  })
_sym_db.RegisterMessage(FlushIndexRequest)

FlushIndexResponse = _reflection.GeneratedProtocolMessageType('FlushIndexResponse', (_message.Message,), {
  'DESCRIPTOR' : _FLUSHINDEXRESPONSE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.FlushIndexResponse)
  })
_sym_db.RegisterMessage(FlushIndexResponse)

Table = _reflection.GeneratedProtocolMessageType('Table', (_message.Message,), {
  'DESCRIPTOR' : _TABLE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.Table)
  })
_sym_db.RegisterMessage(Table)

SQLGetRequest = _reflection.GeneratedProtocolMessageType('SQLGetRequest', (_message.Message,), {
  'DESCRIPTOR' : _SQLGETREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.SQLGetRequest)
  })
_sym_db.RegisterMessage(SQLGetRequest)

VerifiableSQLGetRequest = _reflection.GeneratedProtocolMessageType('VerifiableSQLGetRequest', (_message.Message,), {
  'DESCRIPTOR' : _VERIFIABLESQLGETREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.VerifiableSQLGetRequest)
  })
_sym_db.RegisterMessage(VerifiableSQLGetRequest)

SQLEntry = _reflection.GeneratedProtocolMessageType('SQLEntry', (_message.Message,), {
  'DESCRIPTOR' : _SQLENTRY,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.SQLEntry)
  })
_sym_db.RegisterMessage(SQLEntry)

VerifiableSQLEntry = _reflection.GeneratedProtocolMessageType('VerifiableSQLEntry', (_message.Message,), {

  'ColNamesByIdEntry' : _reflection.GeneratedProtocolMessageType('ColNamesByIdEntry', (_message.Message,), {
    'DESCRIPTOR' : _VERIFIABLESQLENTRY_COLNAMESBYIDENTRY,
    '__module__' : 'schema_pb2'
    # @@protoc_insertion_point(class_scope:immudb.schema.VerifiableSQLEntry.ColNamesByIdEntry)
    })
  ,

  'ColIdsByNameEntry' : _reflection.GeneratedProtocolMessageType('ColIdsByNameEntry', (_message.Message,), {
    'DESCRIPTOR' : _VERIFIABLESQLENTRY_COLIDSBYNAMEENTRY,
    '__module__' : 'schema_pb2'
    # @@protoc_insertion_point(class_scope:immudb.schema.VerifiableSQLEntry.ColIdsByNameEntry)
    })
  ,

  'ColTypesByIdEntry' : _reflection.GeneratedProtocolMessageType('ColTypesByIdEntry', (_message.Message,), {
    'DESCRIPTOR' : _VERIFIABLESQLENTRY_COLTYPESBYIDENTRY,
    '__module__' : 'schema_pb2'
    # @@protoc_insertion_point(class_scope:immudb.schema.VerifiableSQLEntry.ColTypesByIdEntry)
    })
  ,

  'ColLenByIdEntry' : _reflection.GeneratedProtocolMessageType('ColLenByIdEntry', (_message.Message,), {
    'DESCRIPTOR' : _VERIFIABLESQLENTRY_COLLENBYIDENTRY,
    '__module__' : 'schema_pb2'
    # @@protoc_insertion_point(class_scope:immudb.schema.VerifiableSQLEntry.ColLenByIdEntry)
    })
  ,
  'DESCRIPTOR' : _VERIFIABLESQLENTRY,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.VerifiableSQLEntry)
  })
_sym_db.RegisterMessage(VerifiableSQLEntry)
_sym_db.RegisterMessage(VerifiableSQLEntry.ColNamesByIdEntry)
_sym_db.RegisterMessage(VerifiableSQLEntry.ColIdsByNameEntry)
_sym_db.RegisterMessage(VerifiableSQLEntry.ColTypesByIdEntry)
_sym_db.RegisterMessage(VerifiableSQLEntry.ColLenByIdEntry)

UseDatabaseReply = _reflection.GeneratedProtocolMessageType('UseDatabaseReply', (_message.Message,), {
  'DESCRIPTOR' : _USEDATABASEREPLY,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.UseDatabaseReply)
  })
_sym_db.RegisterMessage(UseDatabaseReply)

ChangePermissionRequest = _reflection.GeneratedProtocolMessageType('ChangePermissionRequest', (_message.Message,), {
  'DESCRIPTOR' : _CHANGEPERMISSIONREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.ChangePermissionRequest)
  })
_sym_db.RegisterMessage(ChangePermissionRequest)

SetActiveUserRequest = _reflection.GeneratedProtocolMessageType('SetActiveUserRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETACTIVEUSERREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.SetActiveUserRequest)
  })
_sym_db.RegisterMessage(SetActiveUserRequest)

DatabaseListResponse = _reflection.GeneratedProtocolMessageType('DatabaseListResponse', (_message.Message,), {
  'DESCRIPTOR' : _DATABASELISTRESPONSE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.DatabaseListResponse)
  })
_sym_db.RegisterMessage(DatabaseListResponse)

DatabaseListRequestV2 = _reflection.GeneratedProtocolMessageType('DatabaseListRequestV2', (_message.Message,), {
  'DESCRIPTOR' : _DATABASELISTREQUESTV2,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.DatabaseListRequestV2)
  })
_sym_db.RegisterMessage(DatabaseListRequestV2)

DatabaseListResponseV2 = _reflection.GeneratedProtocolMessageType('DatabaseListResponseV2', (_message.Message,), {
  'DESCRIPTOR' : _DATABASELISTRESPONSEV2,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.DatabaseListResponseV2)
  })
_sym_db.RegisterMessage(DatabaseListResponseV2)

DatabaseWithSettings = _reflection.GeneratedProtocolMessageType('DatabaseWithSettings', (_message.Message,), {
  'DESCRIPTOR' : _DATABASEWITHSETTINGS,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.DatabaseWithSettings)
  })
_sym_db.RegisterMessage(DatabaseWithSettings)

Chunk = _reflection.GeneratedProtocolMessageType('Chunk', (_message.Message,), {
  'DESCRIPTOR' : _CHUNK,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.Chunk)
  })
_sym_db.RegisterMessage(Chunk)

UseSnapshotRequest = _reflection.GeneratedProtocolMessageType('UseSnapshotRequest', (_message.Message,), {
  'DESCRIPTOR' : _USESNAPSHOTREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.UseSnapshotRequest)
  })
_sym_db.RegisterMessage(UseSnapshotRequest)

SQLExecRequest = _reflection.GeneratedProtocolMessageType('SQLExecRequest', (_message.Message,), {
  'DESCRIPTOR' : _SQLEXECREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.SQLExecRequest)
  })
_sym_db.RegisterMessage(SQLExecRequest)

SQLQueryRequest = _reflection.GeneratedProtocolMessageType('SQLQueryRequest', (_message.Message,), {
  'DESCRIPTOR' : _SQLQUERYREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.SQLQueryRequest)
  })
_sym_db.RegisterMessage(SQLQueryRequest)

NamedParam = _reflection.GeneratedProtocolMessageType('NamedParam', (_message.Message,), {
  'DESCRIPTOR' : _NAMEDPARAM,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.NamedParam)
  })
_sym_db.RegisterMessage(NamedParam)

SQLExecResult = _reflection.GeneratedProtocolMessageType('SQLExecResult', (_message.Message,), {
  'DESCRIPTOR' : _SQLEXECRESULT,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.SQLExecResult)
  })
_sym_db.RegisterMessage(SQLExecResult)

CommittedSQLTx = _reflection.GeneratedProtocolMessageType('CommittedSQLTx', (_message.Message,), {

  'LastInsertedPKsEntry' : _reflection.GeneratedProtocolMessageType('LastInsertedPKsEntry', (_message.Message,), {
    'DESCRIPTOR' : _COMMITTEDSQLTX_LASTINSERTEDPKSENTRY,
    '__module__' : 'schema_pb2'
    # @@protoc_insertion_point(class_scope:immudb.schema.CommittedSQLTx.LastInsertedPKsEntry)
    })
  ,

  'FirstInsertedPKsEntry' : _reflection.GeneratedProtocolMessageType('FirstInsertedPKsEntry', (_message.Message,), {
    'DESCRIPTOR' : _COMMITTEDSQLTX_FIRSTINSERTEDPKSENTRY,
    '__module__' : 'schema_pb2'
    # @@protoc_insertion_point(class_scope:immudb.schema.CommittedSQLTx.FirstInsertedPKsEntry)
    })
  ,
  'DESCRIPTOR' : _COMMITTEDSQLTX,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.CommittedSQLTx)
  })
_sym_db.RegisterMessage(CommittedSQLTx)
_sym_db.RegisterMessage(CommittedSQLTx.LastInsertedPKsEntry)
_sym_db.RegisterMessage(CommittedSQLTx.FirstInsertedPKsEntry)

SQLQueryResult = _reflection.GeneratedProtocolMessageType('SQLQueryResult', (_message.Message,), {
  'DESCRIPTOR' : _SQLQUERYRESULT,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.SQLQueryResult)
  })
_sym_db.RegisterMessage(SQLQueryResult)

Column = _reflection.GeneratedProtocolMessageType('Column', (_message.Message,), {
  'DESCRIPTOR' : _COLUMN,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.Column)
  })
_sym_db.RegisterMessage(Column)

Row = _reflection.GeneratedProtocolMessageType('Row', (_message.Message,), {
  'DESCRIPTOR' : _ROW,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.Row)
  })
_sym_db.RegisterMessage(Row)

SQLValue = _reflection.GeneratedProtocolMessageType('SQLValue', (_message.Message,), {
  'DESCRIPTOR' : _SQLVALUE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.SQLValue)
  })
_sym_db.RegisterMessage(SQLValue)

NewTxRequest = _reflection.GeneratedProtocolMessageType('NewTxRequest', (_message.Message,), {
  'DESCRIPTOR' : _NEWTXREQUEST,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.NewTxRequest)
  })
_sym_db.RegisterMessage(NewTxRequest)

NewTxResponse = _reflection.GeneratedProtocolMessageType('NewTxResponse', (_message.Message,), {
  'DESCRIPTOR' : _NEWTXRESPONSE,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.NewTxResponse)
  })
_sym_db.RegisterMessage(NewTxResponse)

ErrorInfo = _reflection.GeneratedProtocolMessageType('ErrorInfo', (_message.Message,), {
  'DESCRIPTOR' : _ERRORINFO,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.ErrorInfo)
  })
_sym_db.RegisterMessage(ErrorInfo)

DebugInfo = _reflection.GeneratedProtocolMessageType('DebugInfo', (_message.Message,), {
  'DESCRIPTOR' : _DEBUGINFO,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.DebugInfo)
  })
_sym_db.RegisterMessage(DebugInfo)

RetryInfo = _reflection.GeneratedProtocolMessageType('RetryInfo', (_message.Message,), {
  'DESCRIPTOR' : _RETRYINFO,
  '__module__' : 'schema_pb2'
  # @@protoc_insertion_point(class_scope:immudb.schema.RetryInfo)
  })
_sym_db.RegisterMessage(RetryInfo)


DESCRIPTOR._options = None
_VERIFIABLESQLENTRY_COLNAMESBYIDENTRY._options = None
_VERIFIABLESQLENTRY_COLIDSBYNAMEENTRY._options = None
_VERIFIABLESQLENTRY_COLTYPESBYIDENTRY._options = None
_VERIFIABLESQLENTRY_COLLENBYIDENTRY._options = None
_COMMITTEDSQLTX_LASTINSERTEDPKSENTRY._options = None
_COMMITTEDSQLTX_FIRSTINSERTEDPKSENTRY._options = None

_IMMUSERVICE = _descriptor.ServiceDescriptor(
  name='ImmuService',
  full_name='immudb.schema.ImmuService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=12444,
  serialized_end=17566,
  methods=[
  _descriptor.MethodDescriptor(
    name='ListUsers',
    full_name='immudb.schema.ImmuService.ListUsers',
    index=0,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=_USERLIST,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CreateUser',
    full_name='immudb.schema.ImmuService.CreateUser',
    index=1,
    containing_service=None,
    input_type=_CREATEUSERREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ChangePassword',
    full_name='immudb.schema.ImmuService.ChangePassword',
    index=2,
    containing_service=None,
    input_type=_CHANGEPASSWORDREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ChangePermission',
    full_name='immudb.schema.ImmuService.ChangePermission',
    index=3,
    containing_service=None,
    input_type=_CHANGEPERMISSIONREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='SetActiveUser',
    full_name='immudb.schema.ImmuService.SetActiveUser',
    index=4,
    containing_service=None,
    input_type=_SETACTIVEUSERREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateAuthConfig',
    full_name='immudb.schema.ImmuService.UpdateAuthConfig',
    index=5,
    containing_service=None,
    input_type=_AUTHCONFIG,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateMTLSConfig',
    full_name='immudb.schema.ImmuService.UpdateMTLSConfig',
    index=6,
    containing_service=None,
    input_type=_MTLSCONFIG,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='OpenSession',
    full_name='immudb.schema.ImmuService.OpenSession',
    index=7,
    containing_service=None,
    input_type=_OPENSESSIONREQUEST,
    output_type=_OPENSESSIONRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CloseSession',
    full_name='immudb.schema.ImmuService.CloseSession',
    index=8,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='KeepAlive',
    full_name='immudb.schema.ImmuService.KeepAlive',
    index=9,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='NewTx',
    full_name='immudb.schema.ImmuService.NewTx',
    index=10,
    containing_service=None,
    input_type=_NEWTXREQUEST,
    output_type=_NEWTXRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Commit',
    full_name='immudb.schema.ImmuService.Commit',
    index=11,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=_COMMITTEDSQLTX,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Rollback',
    full_name='immudb.schema.ImmuService.Rollback',
    index=12,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='TxSQLExec',
    full_name='immudb.schema.ImmuService.TxSQLExec',
    index=13,
    containing_service=None,
    input_type=_SQLEXECREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='TxSQLQuery',
    full_name='immudb.schema.ImmuService.TxSQLQuery',
    index=14,
    containing_service=None,
    input_type=_SQLQUERYREQUEST,
    output_type=_SQLQUERYRESULT,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Login',
    full_name='immudb.schema.ImmuService.Login',
    index=15,
    containing_service=None,
    input_type=_LOGINREQUEST,
    output_type=_LOGINRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Logout',
    full_name='immudb.schema.ImmuService.Logout',
    index=16,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Set',
    full_name='immudb.schema.ImmuService.Set',
    index=17,
    containing_service=None,
    input_type=_SETREQUEST,
    output_type=_TXHEADER,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='VerifiableSet',
    full_name='immudb.schema.ImmuService.VerifiableSet',
    index=18,
    containing_service=None,
    input_type=_VERIFIABLESETREQUEST,
    output_type=_VERIFIABLETX,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Get',
    full_name='immudb.schema.ImmuService.Get',
    index=19,
    containing_service=None,
    input_type=_KEYREQUEST,
    output_type=_ENTRY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='VerifiableGet',
    full_name='immudb.schema.ImmuService.VerifiableGet',
    index=20,
    containing_service=None,
    input_type=_VERIFIABLEGETREQUEST,
    output_type=_VERIFIABLEENTRY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Delete',
    full_name='immudb.schema.ImmuService.Delete',
    index=21,
    containing_service=None,
    input_type=_DELETEKEYSREQUEST,
    output_type=_TXHEADER,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetAll',
    full_name='immudb.schema.ImmuService.GetAll',
    index=22,
    containing_service=None,
    input_type=_KEYLISTREQUEST,
    output_type=_ENTRIES,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ExecAll',
    full_name='immudb.schema.ImmuService.ExecAll',
    index=23,
    containing_service=None,
    input_type=_EXECALLREQUEST,
    output_type=_TXHEADER,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Scan',
    full_name='immudb.schema.ImmuService.Scan',
    index=24,
    containing_service=None,
    input_type=_SCANREQUEST,
    output_type=_ENTRIES,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Count',
    full_name='immudb.schema.ImmuService.Count',
    index=25,
    containing_service=None,
    input_type=_KEYPREFIX,
    output_type=_ENTRYCOUNT,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CountAll',
    full_name='immudb.schema.ImmuService.CountAll',
    index=26,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=_ENTRYCOUNT,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='TxById',
    full_name='immudb.schema.ImmuService.TxById',
    index=27,
    containing_service=None,
    input_type=_TXREQUEST,
    output_type=_TX,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='VerifiableTxById',
    full_name='immudb.schema.ImmuService.VerifiableTxById',
    index=28,
    containing_service=None,
    input_type=_VERIFIABLETXREQUEST,
    output_type=_VERIFIABLETX,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='TxScan',
    full_name='immudb.schema.ImmuService.TxScan',
    index=29,
    containing_service=None,
    input_type=_TXSCANREQUEST,
    output_type=_TXLIST,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='History',
    full_name='immudb.schema.ImmuService.History',
    index=30,
    containing_service=None,
    input_type=_HISTORYREQUEST,
    output_type=_ENTRIES,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Health',
    full_name='immudb.schema.ImmuService.Health',
    index=31,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=_HEALTHRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DatabaseHealth',
    full_name='immudb.schema.ImmuService.DatabaseHealth',
    index=32,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=_DATABASEHEALTHRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CurrentState',
    full_name='immudb.schema.ImmuService.CurrentState',
    index=33,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=_IMMUTABLESTATE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='SetReference',
    full_name='immudb.schema.ImmuService.SetReference',
    index=34,
    containing_service=None,
    input_type=_REFERENCEREQUEST,
    output_type=_TXHEADER,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='VerifiableSetReference',
    full_name='immudb.schema.ImmuService.VerifiableSetReference',
    index=35,
    containing_service=None,
    input_type=_VERIFIABLEREFERENCEREQUEST,
    output_type=_VERIFIABLETX,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ZAdd',
    full_name='immudb.schema.ImmuService.ZAdd',
    index=36,
    containing_service=None,
    input_type=_ZADDREQUEST,
    output_type=_TXHEADER,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='VerifiableZAdd',
    full_name='immudb.schema.ImmuService.VerifiableZAdd',
    index=37,
    containing_service=None,
    input_type=_VERIFIABLEZADDREQUEST,
    output_type=_VERIFIABLETX,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ZScan',
    full_name='immudb.schema.ImmuService.ZScan',
    index=38,
    containing_service=None,
    input_type=_ZSCANREQUEST,
    output_type=_ZENTRIES,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CreateDatabase',
    full_name='immudb.schema.ImmuService.CreateDatabase',
    index=39,
    containing_service=None,
    input_type=_DATABASE,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CreateDatabaseWith',
    full_name='immudb.schema.ImmuService.CreateDatabaseWith',
    index=40,
    containing_service=None,
    input_type=_DATABASESETTINGS,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CreateDatabaseV2',
    full_name='immudb.schema.ImmuService.CreateDatabaseV2',
    index=41,
    containing_service=None,
    input_type=_CREATEDATABASEREQUEST,
    output_type=_CREATEDATABASERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='LoadDatabase',
    full_name='immudb.schema.ImmuService.LoadDatabase',
    index=42,
    containing_service=None,
    input_type=_LOADDATABASEREQUEST,
    output_type=_LOADDATABASERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UnloadDatabase',
    full_name='immudb.schema.ImmuService.UnloadDatabase',
    index=43,
    containing_service=None,
    input_type=_UNLOADDATABASEREQUEST,
    output_type=_UNLOADDATABASERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteDatabase',
    full_name='immudb.schema.ImmuService.DeleteDatabase',
    index=44,
    containing_service=None,
    input_type=_DELETEDATABASEREQUEST,
    output_type=_DELETEDATABASERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DatabaseList',
    full_name='immudb.schema.ImmuService.DatabaseList',
    index=45,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=_DATABASELISTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DatabaseListV2',
    full_name='immudb.schema.ImmuService.DatabaseListV2',
    index=46,
    containing_service=None,
    input_type=_DATABASELISTREQUESTV2,
    output_type=_DATABASELISTRESPONSEV2,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UseDatabase',
    full_name='immudb.schema.ImmuService.UseDatabase',
    index=47,
    containing_service=None,
    input_type=_DATABASE,
    output_type=_USEDATABASEREPLY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateDatabase',
    full_name='immudb.schema.ImmuService.UpdateDatabase',
    index=48,
    containing_service=None,
    input_type=_DATABASESETTINGS,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateDatabaseV2',
    full_name='immudb.schema.ImmuService.UpdateDatabaseV2',
    index=49,
    containing_service=None,
    input_type=_UPDATEDATABASEREQUEST,
    output_type=_UPDATEDATABASERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetDatabaseSettings',
    full_name='immudb.schema.ImmuService.GetDatabaseSettings',
    index=50,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=_DATABASESETTINGS,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetDatabaseSettingsV2',
    full_name='immudb.schema.ImmuService.GetDatabaseSettingsV2',
    index=51,
    containing_service=None,
    input_type=_DATABASESETTINGSREQUEST,
    output_type=_DATABASESETTINGSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='FlushIndex',
    full_name='immudb.schema.ImmuService.FlushIndex',
    index=52,
    containing_service=None,
    input_type=_FLUSHINDEXREQUEST,
    output_type=_FLUSHINDEXRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CompactIndex',
    full_name='immudb.schema.ImmuService.CompactIndex',
    index=53,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='streamGet',
    full_name='immudb.schema.ImmuService.streamGet',
    index=54,
    containing_service=None,
    input_type=_KEYREQUEST,
    output_type=_CHUNK,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='streamSet',
    full_name='immudb.schema.ImmuService.streamSet',
    index=55,
    containing_service=None,
    input_type=_CHUNK,
    output_type=_TXHEADER,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='streamVerifiableGet',
    full_name='immudb.schema.ImmuService.streamVerifiableGet',
    index=56,
    containing_service=None,
    input_type=_VERIFIABLEGETREQUEST,
    output_type=_CHUNK,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='streamVerifiableSet',
    full_name='immudb.schema.ImmuService.streamVerifiableSet',
    index=57,
    containing_service=None,
    input_type=_CHUNK,
    output_type=_VERIFIABLETX,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='streamScan',
    full_name='immudb.schema.ImmuService.streamScan',
    index=58,
    containing_service=None,
    input_type=_SCANREQUEST,
    output_type=_CHUNK,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='streamZScan',
    full_name='immudb.schema.ImmuService.streamZScan',
    index=59,
    containing_service=None,
    input_type=_ZSCANREQUEST,
    output_type=_CHUNK,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='streamHistory',
    full_name='immudb.schema.ImmuService.streamHistory',
    index=60,
    containing_service=None,
    input_type=_HISTORYREQUEST,
    output_type=_CHUNK,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='streamExecAll',
    full_name='immudb.schema.ImmuService.streamExecAll',
    index=61,
    containing_service=None,
    input_type=_CHUNK,
    output_type=_TXHEADER,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='exportTx',
    full_name='immudb.schema.ImmuService.exportTx',
    index=62,
    containing_service=None,
    input_type=_EXPORTTXREQUEST,
    output_type=_CHUNK,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='replicateTx',
    full_name='immudb.schema.ImmuService.replicateTx',
    index=63,
    containing_service=None,
    input_type=_CHUNK,
    output_type=_TXHEADER,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='SQLExec',
    full_name='immudb.schema.ImmuService.SQLExec',
    index=64,
    containing_service=None,
    input_type=_SQLEXECREQUEST,
    output_type=_SQLEXECRESULT,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='SQLQuery',
    full_name='immudb.schema.ImmuService.SQLQuery',
    index=65,
    containing_service=None,
    input_type=_SQLQUERYREQUEST,
    output_type=_SQLQUERYRESULT,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ListTables',
    full_name='immudb.schema.ImmuService.ListTables',
    index=66,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=_SQLQUERYRESULT,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DescribeTable',
    full_name='immudb.schema.ImmuService.DescribeTable',
    index=67,
    containing_service=None,
    input_type=_TABLE,
    output_type=_SQLQUERYRESULT,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='VerifiableSQLGet',
    full_name='immudb.schema.ImmuService.VerifiableSQLGet',
    index=68,
    containing_service=None,
    input_type=_VERIFIABLESQLGETREQUEST,
    output_type=_VERIFIABLESQLENTRY,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_IMMUSERVICE)

DESCRIPTOR.services_by_name['ImmuService'] = _IMMUSERVICE

# @@protoc_insertion_point(module_scope)
